#pragma once

/******************************************************************************
 CNBrainPan.h

	Interface for the NBrainPan Class
	
	Source code for "NervousSystem"
	Copyright  1994 Daniel McFarlane. All rights reserved.
	All use is dictated by the terms of the "GNU General Public 
	License" (see the file "CNObjectView.c" for details).
	
	This class is the workspace of NervousSystem.  An instance of this
	class is attached to all NervousSystem documents.  
	
	It is a scrollable
	panorama, where all instances of CNObjectView are tacked onto it, or 
	"fixed" to it.  So that when "this" is scrolled or moved or size-changed,
	all the instances of CNObjectView affixed to it behave accordingly.
	
	All interactive creation of objects is handled here.  this::DoClick 
	manages initializing the appropriate creation "Tasks" and sending the tasks
	the "TrackMouse()" messages.
	
	Several of the methods defined for this class have to do with managing 
	"itsSelection".  "itsSelection" is an instance of CNOViewOwner used to 
	refer to an arbitrary subsed of the objects enclosed by "this".
	"itsSelection" is crucial for dragging, and cut/copy/paste and undo/redo.
 ******************************************************************************/
 
#define _H_CNBrainPan

#include "CPanorama.h"					/* Interface for its superclass		*/
#include "LongCoordinates.h"

class CNOViewOwner;
class CNObjectView;
class CList;
class CNervousSystemDoc;


class CNBrainPan : public CPanorama {	/* Class Declaration				*/

public:
								/** Instance Variables **/
	LongRect myDataExtent;

	// The instances of CNObjectView:: that are currently selected.
	CNOViewOwner 	*itsSelection;

								/** Instance Methods **/
	void	IViewTemp	(CView			*anEnclosure,
						 CBureaucrat	*aSupervisor,
						 Ptr			viewData);		// is override
	void	INBrainPan(CView *anEnclosure, CBureaucrat	*aSupervisor, 
					short aWidth, short aHeight, short aHEncl, short aVEncl, 
					SizingOption aHSizing, SizingOption aVSizing);
	void	Dispose(void);
	void	DoClick(Point hitPt, short modifierKeys, long when);
	void	ChangeSize(Rect *delta, Boolean redraw);
	void	AdjustBounds(void);
	void	CalcMyDataExtent(void);	
	void	ChangeMyDataExtent(LongPt *currLPt);	
	void	Draw(Rect *area);			// is override
	void	DoCommand(long theCommand);
	void	UpdateMenus	(void);
	void	EncloseWith(CNObjectView *aSubview);
	void	ShrinkWrapWithout(CNObjectView *aSubview);
	void	ObtainTraveler(CNObjectView **aNOView);
	Boolean	IncludesSubview(CNObjectView *aNOView);
	CNervousSystemDoc	*GetMyNervousSysDoc(void);
	
	// These methods are for managing "itsSelection".
	void	ResetSelectionList(void);
	void	AddToSelection(CNObjectView *newChosen);
	void	RemoveFromSelection(CNObjectView *newUnChosen);
	void	UnionWithSelection(CNOViewOwner *aNOVOwner);
	void	NotWithSelection(CNOViewOwner *aNOVOwner);
	void	RemoveNOViews(CNOViewOwner *aNOVOwner);
	void	AddNOVObjects(CNOViewOwner *aNOVOwner);
	CNOViewOwner	*CopyItsSelection(CBureaucrat *aSupervisor);
	void	AddSubview(CView *theSubview);
	Boolean	IsBeingDisposed(void);
	CList	*GetSubviews(void);
	
private:
	void	InitMyDataExtent(void);
	Boolean	fBeingDisposed;
};

