/******************************************************************************
 CNDendriteView.c

	The NDendriteView Class
		
	SUPERCLASS = CNObjectView
	
	Source code for "NervousSystem"
	Copyright  1994 Daniel McFarlane. All rights reserved.
	All use is dictated by the terms of the "GNU General Public 
	License" (see the file "CNObjectView.c" for details).
 ******************************************************************************/

#include "Global.h"
#include "CNDendriteView.h"
#include <stdio.h>
#include "LongQD.h"
#include "CNOViewTask.h"
#include "MainWindow.h"
#include "CArray.h"
#include "CNDendriteModel.h"
#include "DendriteController.h"
#include "NervousSystemDoc.h"



/******************************************************************************
 INDendriteView

		Initialize a NDendriteView object
 ******************************************************************************/

void	CNDendriteView::INDendriteView(
	CView			*anEnclosure,
	CBureaucrat		*aSupervisor,
	short			aWidth,
	short			aHeight,
	short			aHEncl,
	short			aVEncl,
	SizingOption	aHSizing,
	SizingOption	aVSizing,
	long 			aHFrameOrigin, 
	long 			aVFrameOrigin, 
	CNDendriteModel		*aNDendriteModel)
{
	CNObjectView::INObjectView(anEnclosure, aSupervisor,
					aWidth, aHeight, aHEncl, aVEncl, aHSizing, aVSizing, 
					aHFrameOrigin, aVFrameOrigin);
	
	// Initialize the instance of the NModelElem managed by this view.
	if (aNDendriteModel)
		myNModelElem = aNDendriteModel->CopyModel();
	else {
		myNModelElem = new(CNDendriteModel);
		((CNDendriteModel *) myNModelElem)->INDendriteModel();
	}
	
	wantsClicks = TRUE;
	SetCanBeGopher(TRUE);
	
	// Initialize Graphic Data.
	itsGraphicData = NULL;
	
	penWidth = BRUSH_WIDTH_DENDRITE;
}


/******************************************************************************
 CopyNOViewInto {OVERRIDE}
 
******************************************************************************/

CNObjectView *CNDendriteView::CopyNOViewInto(CView *anEnclosure, CBureaucrat *aSupervisor)
{
	CNDendriteView		*theCopy = NULL;
	
	// Create a new CNObjectView as a copy.
	theCopy = new(CNDendriteView);

	// Initialize the new object to be a copy of "this".
	theCopy->INDendriteView(anEnclosure, aSupervisor, width, height, 
				hEncl, vEncl, hSizing, vSizing, hFrameOrigin, vFrameOrigin, 
				((CNDendriteModel *) myNModelElem));
	
	INOViewCopy(theCopy);
	
	return theCopy;
}


/******************************************************************************
 INOViewCopy 
 
 	Initialize a copy of this NOView.
******************************************************************************/

void CNDendriteView::INOViewCopy(CNObjectView *theCopy)
{
	((CNDendriteView *) theCopy)->itsGraphicData = (CArray *) itsGraphicData->Copy();
	
	inherited::INOViewCopy(theCopy);
}


/******************************************************************************
 DoClick {OVERRIDE}

		Mouse click inside the NBrainPan
 ******************************************************************************/

void	CNDendriteView::DoClick(
	Point		hitPt,
	short		modifierKeys,
	long		when)
{
	inherited::DoClick(hitPt, modifierKeys, when);
}
	

/******************************************************************************
 PrintGDataArray 

 ******************************************************************************/

void	CNDendriteView::PrintGDataArray(void) {
	long	i;
	LongPt	nextPt;
	long	numItems;

	numItems = itsGraphicData->GetNumItems();
	printf("\nThere are %ld items in the array. i,h,v.\n", numItems);
	for (i = 1; i <= numItems; ++i) {
		itsGraphicData->GetItem(&nextPt, i);
		printf("%ld\t%ld\t%ld\n", i, nextPt.h, nextPt.v);
	}
	printf("\n");
}


/******************************************************************************
 Dispose 

		Dispose of a NDendriteView
 ******************************************************************************/

void	CNDendriteView::Dispose()
{
	if (itsGraphicData)
		itsGraphicData->Dispose();	/* Throw out its graphical model.	*/

		/* makes sure that the last undo task is disposed. Since the 	*/
		/* undo tasks usually have references back to this pane, 		*/
		/* it would be bad if this pane was disposed and the task was 	*/
		/* not informed.												*/

	inherited::Dispose();			/* Pass message on to superclass	*/
}


/******************************************************************************
 DoDialog   
 
 ******************************************************************************/

void	CNDendriteView::DoDialog(void)
{
	CNervousSystemDoc	*aNervousSystemDoc = NULL;
	
	aNervousSystemDoc = GetMyNervousSysDoc();
	DoDendriteController(aNervousSystemDoc, ((CNDendriteModel *) myNModelElem));
}









