/******************************************************************************
 CNHotSpotView.c

	The NHotSpotView Class
		
	SUPERCLASS = CNObjectView
	
	Source code for "NervousSystem"
	Copyright  1994 Daniel McFarlane. All rights reserved.
	All use is dictated by the terms of the "GNU General Public 
	License" (see the file "CNObjectView.c" for details).
 ******************************************************************************/

#include "Global.h"
#include "CNHotSpotView.h"
#include "Constants.h"      
#include "LongQD.h"
#include "LongCoordinates.h"
#include "CView.h"
#include "CBureaucrat.h"
#include "CNObjectView.h"

	
/**** C O N S T R U C T I O N / D E S T R U C T I O N   M E T H O D S ****/


/******************************************************************************
 INHotSpotView

		Initialize a NHotSpotView object
 ******************************************************************************/

void	CNHotSpotView::INHotSpotView(
	CView			*anEnclosure,
	CBureaucrat		*aSupervisor,
	short			aWidth,
	short			aHeight,
	short			aHEncl,
	short			aVEncl,
	SizingOption	aHSizing,
	SizingOption	aVSizing,
	long 			aHFrameOrigin, 
	long 			aVFrameOrigin)
{
	Rect 		boundingRect;

	// Do the unique initialization.
	myWidth = aWidth;
	
	// Do the inherited initialization.
	CNObjectView::INObjectView(anEnclosure, aSupervisor, 
				aWidth, aHeight, aHEncl, aVEncl, aHSizing, aVSizing, 
				aHFrameOrigin, aVFrameOrigin);
					
	// HotSpots are not initially visible.
	Hide();
	
	// Initialize my region, and my pane & view parameters.
	InitMyRegion();
}
	

/******************************************************************************
 INOViewCopy 
 
 	Initialize a copy of this NOView.
******************************************************************************/

void CNHotSpotView::INOViewCopy(CNObjectView *theCopy)
{
	inherited::INOViewCopy(theCopy);
}


/******************************************************************************
 CopyNOViewInto {OVERRIDE}
 
******************************************************************************/

CNObjectView *CNHotSpotView::CopyNOViewInto(CView *anEnclosure, CBureaucrat *aSupervisor)
{
	CNHotSpotView		*theCopy = NULL;
	
	// Create a new CNObjectView as a copy.
	theCopy = new(CNHotSpotView);

	// Initialize the new object to be a copy of "this".
	theCopy->INHotSpotView(anEnclosure, aSupervisor, width, height, 
				hEncl, vEncl, hSizing, vSizing, hFrameOrigin, vFrameOrigin);
	
	INOViewCopy(theCopy);
	
	return theCopy;
}


/******************************************************************************
 SetFDrawHotSpot

	This method does not initiate a Refresh of this object, it relies on the 
	calling environment to manage the necessary graphical update.
  ******************************************************************************/

void	CNHotSpotView::SetFDrawHotSpot(Boolean fDraw)
{		
	if (fDraw)
		Show();
	else
		Hide();
		
	inherited::SetFDrawHotSpot(fDraw);
}


/******************************************************************************
 Draw

 ******************************************************************************/

void	CNHotSpotView::Draw(Rect *area)
{
	PenState	savePen;

	if ((fDrawHotSpots) && (ReallyVisible())) {
		// Initialize the Drawing environment.
		Prepare();
	
		// Draw "this"
		FrameRgn(itsRegion);
	}
}
	

/******************************************************************************
 Dawdle {OVERRIDE}

		Animate the NHotSpotView during Idle time
 ******************************************************************************/

void	CNHotSpotView::Dawdle(
	long	*maxSleep)
{
	Pattern		thePat;
	long		ticks;
	Rect		drawArea;
	LongRect	drawAreaLong;
	
	if (IsVisible()) {
		GetAperture(&drawAreaLong);
		LongToQDRect(&drawAreaLong, &drawArea);

		Prepare();
		
		PenNormal();				/* Draw rectangle with one pattern		*/
		GetIndPattern( thePat, PAT_MARCH_ANTS, 1);
		PenPat(thePat);
		Draw(&drawArea);
		
		Delay(6, &ticks);			/* Wait a while							*/
			
									/* Redraw with a pattern offset from	*/
									/*   the first. This gives the			*/
									/*   illusion of marching ants.			*/
		GetIndPattern( thePat, PAT_MARCH_ANTS, 2);
		PenPat(thePat);
		Draw(&drawArea);
		*maxSleep = 5;
	}
}
	

/******************************************************************************
 InitMyRegion

 ******************************************************************************/

void	CNHotSpotView::InitMyRegion(void)
{
	LongRect	circleBounds;
	
	// Initialize circleBounds.
	SetLongRect(&circleBounds, 0, 0, myWidth, myWidth);
	
	Prepare();
	
	OpenRgn();
	
	LFrameOval(&circleBounds);

	CloseRgn(itsRegion);
}


/******************************************************************************
 GetWidth

 ******************************************************************************/

long	CNHotSpotView::GetWidth(void)
{
	return myWidth;
}




