/******************************************************************************
 CNNetworkView.c
	
	The NNetworkView Class
	
	SUPERCLASS = CNOVCollection
	
	Source code for "NervousSystem"
	Copyright  1994 Daniel McFarlane. All rights reserved.
	All use is dictated by the terms of the "GNU General Public 
	License" (see the file "CNObjectView.c" for details).
 ******************************************************************************/

#include "Global.h"
#include "CNNetworkView.h"
#include <stdio.h>
#include <string.h>
#include "CList.h"
#include <QuickDraw.h>
#include "CNOVDraggerTask.h"
#include "CNMyGlobals.h"
#include "CNOViewOwner.h"
#include "CPaneBorder.h"
#include "CNCompositeModel.h"
#include "CNHotSpotView.h"
#include "CNNeuronView.h"


// External global variables used here.
extern short gClicks;


/******************************************************************************
 INNetworkView

		Initialize a NNetworkView object
 ******************************************************************************/

void	CNNetworkView::INNetworkView(
	CView			*anEnclosure,
	CBureaucrat		*aSupervisor,
	short			aWidth,
	short			aHeight,
	short			aHEncl,
	short			aVEncl,
	SizingOption	aHSizing,
	SizingOption	aVSizing,
	long 			aHFrameOrigin, 
	long			aVFrameOrigin, 
	CNNetworkModel	*aNNetworkModel)
{
	CNOVCollection::INOVCollection(anEnclosure, aSupervisor,
					aWidth, aHeight, aHEncl, aVEncl, aHSizing, aVSizing, 
					aHFrameOrigin, aVFrameOrigin);
		
	// Initialize the instance of the NModelElem managed by this view.
	if (aNNetworkModel)
		myNModelElem = aNNetworkModel->CopyModel();
	else {
		myNModelElem = new(CNNetworkModel);
		((CNNetworkModel *) myNModelElem)->INNetworkModel();
	}
}


/******************************************************************************
 SetCopiedModelsIdenticalToOriginals  		{behavior is specific to children}
 
******************************************************************************/

void CNNetworkView::SetCopiedModelsIdenticalToOriginals(
	CNObjectView *aNOView, 
	CNObjectView *aCopiedNOView)
{
	long			i;
	CNObjectView 	*nextObjectSubview = NULL;
	CNObjectView 	*nextCopiedObjectSubview = NULL;
	long			numObjectSubviews;
	long			numCopiedObjectSubviews;
	CNModel			*nextModel = NULL;
	CNModel			*nextCopiedModel = NULL;
	
	if ((aNOView) && (aCopiedNOView)) {
	
		//INV: both aNOView and aCopiedNOView are CNNeuronView's.
		
		numObjectSubviews = (aNOView->itsSubviews)->GetNumItems();
		numCopiedObjectSubviews = (aCopiedNOView->itsSubviews)->GetNumItems();
		if (numObjectSubviews != numCopiedObjectSubviews) return;
		
		for (i = 1; i <= numObjectSubviews; ++i) {
		
			// Access the next object subviews in both the original and the copied.
			nextObjectSubview = (CNObjectView *) (aNOView->itsSubviews)->NthItem(i);
			nextCopiedObjectSubview = (CNObjectView *) (aCopiedNOView->itsSubviews)->NthItem(i);
		
			// Access the models of these next object subviews.
			nextModel = nextObjectSubview->GetModel();
			nextCopiedModel = nextCopiedObjectSubview->GetModel();
			
			// Set the copied model to be identical to the original model.
			nextCopiedModel->MakeIdentical(nextModel);
		}
	}
}


/******************************************************************************
 FindHSpotWithinTraveledObj  		{this is an override}
 
******************************************************************************/

void CNNetworkView::FindHSpotWithinTraveledObj(
	CNObjectView *aNOView, 
	CNObjectView *aCopiedNOView, 
	CNHotSpotView **aNHSpot)
{
	int		objectSubviewIndex = -1;
	int		neuronSubviewIndex = -1;
	CNObjectView	*anObjectView = NULL;
	CNNeuronView	*aNeuronView = NULL;
	CNObjectView	*aTraveledObjectView = NULL;
	CNHotSpotView	*aTraveledHotSpotView = NULL;
	
	if ((aNOView) && (aCopiedNOView) && (*aNHSpot)) {
		
		// Initialize CNObjectView information.
		anObjectView = (CNObjectView *) (*aNHSpot)->itsEnclosure;
		objectSubviewIndex = anObjectView->GetIndexOfSubview((*aNHSpot));
		
		// Initialize CNNeuronView information.
		aNeuronView = (CNNeuronView *) anObjectView->itsEnclosure;
		if (aNeuronView != ((CNNeuronView *) aNOView)) return;
		neuronSubviewIndex = aNeuronView->GetIndexOfSubview(anObjectView);
		//if (((CNNetworkView *) aNeuronView->itsEnclosure) != this) return;
		
		// Access the corresponding CNObjectView within the copy.
		aTraveledObjectView = (CNObjectView *) (aCopiedNOView->itsSubviews)->
				NthItem(neuronSubviewIndex);
		
		// Access the corresponding CNHotSpotView within the copy.
		aTraveledHotSpotView = (CNHotSpotView *) (aTraveledObjectView->itsSubviews)->
				NthItem(objectSubviewIndex);
		
		// Make the assignment;
		(*aNHSpot) = aTraveledHotSpotView;
	}
}


/******************************************************************************
 CopyNOViewInto {OVERRIDE}
 
******************************************************************************/

CNObjectView *CNNetworkView::CopyNOViewInto(CView *anEnclosure, CBureaucrat *aSupervisor)
{
	CNNetworkView		*theCopy = NULL;
	
	// Create a new CNObjectView as a copy.
	theCopy = new(CNNetworkView);

	// Initialize the new object to be a copy of "this".
	theCopy->INNetworkView(anEnclosure, aSupervisor, width, height, 
				hEncl, vEncl, hSizing, vSizing, hFrameOrigin, vFrameOrigin, 
				((CNNetworkModel *) myNModelElem));
	
	// Initialize a flag saying that "this" is being copied.  This flag is used
	// in calls to Enclose with to know whether subviews need to be moved into 
	fBeingCopied = TRUE;
	theCopy->fBeingCopied = TRUE;

	INOViewCopy(theCopy);
	
	INModelCopy(theCopy);
		
	// Copying completed.
	fBeingCopied = FALSE;
	theCopy->fBeingCopied = FALSE;
	
	return theCopy;
}


/******************************************************************************
 INModelCopy 
 
 	Initialize the model of a copy of this NOView.
	Copy all the connections from the original model of this object.
******************************************************************************/

void CNNetworkView::INModelCopy(CNObjectView *theCopy)
{
	(theCopy->myNModelElem)->MakeConnections(theCopy->BuildListOfModelsOfSubviews());
}


/******************************************************************************
 INOViewCopy 
 
 	Initialize a copy of this NOView.
******************************************************************************/

void CNNetworkView::INOViewCopy(CNObjectView *theCopy)
{
	inherited::INOViewCopy(theCopy);
}


/******************************************************************************
 Dispose 

		Dispose of a NNetworkView
 ******************************************************************************/

void	CNNetworkView::Dispose()
{
		/* makes sure that the last undo task is disposed. Since the 	*/
		/* undo tasks usually have references back to this pane, 		*/
		/* it would be bad if this pane was disposed and the task was 	*/
		/* not informed.												*/

	inherited::Dispose();			/* Pass message on to superclass	*/
}


