/******************************************************************************
 CNOVConnectTask.h

	Interface for the NOVConnectTask Class
	
	Copyright  1993 by Daniel McFarlane. All rights reserved.

	Instances of this class are called from CNOVDraggerTask.  CNOVDraggerTask
	discovers whether a connection is needed and then initializes and sends
	"this" "Do()".
	
	After two objects are connected the proper enclosing objects are managed
	so that the connected objects become part of a larger CNNeuronView and possibly
	also part of a larger CNNetworkView.  Once objects are connected they 
	behave as part of a group for future dragging and cut/copy/paste.
	
	Note: "Undo" and "Redo" are partially defined, but not yet operational.
	This implementation is left for a future project.
 ******************************************************************************/
 
#define _H_CNOVConnectTask

#include "CTask.h"					/* Interface for its superclass		*/
#include "CNMyGlobals.h"

class CNObjectView;
class CNBrainPan;
class CNHotSpotView;
class CNModel;
class CNNeuronView;


typedef enum {noPair = -1, netNet = 0, netNeu, netObj, neuNet, neuNeu, neuObj, objNet, 
objNeu, objObj} connectKindType;


class CNOVConnectTask : public CTask {			/* Class Declaration				*/

public:
								/** Instance Methods **/
	void		INOVConnectTask(CNHotSpotView *aNHotSpotFixed, CNHotSpotView *aNHotSpotDragged, 
						CNBrainPan *aNBrainPan);
	
	virtual	void	Dispose(void);
	virtual void	Do(void);
	virtual void	DoForReal(void);
	virtual void	Undo(void);
	virtual void	Redo(void);
	virtual CNHotSpotView	*GetDraggedHSpot(void);
	virtual CNHotSpotView	*GetFixedHSpot(void);
	
private:
	CNBrainPan			*itsNBrainPan;
	connectKindType		connectKind;
	CNHotSpotView		*itsNHotSpotFixed;
	CNHotSpotView		*itsNHotSpotDragged;
	CNObjectView		*fixedTopEncl;
	CNObjectView		*draggedTopEncl;
	CNNeuronView		*fixedNeuron;
	CNNeuronView		*draggedNeuron;
	void	ConnectNetNet(void);
	void	ConnectNetNeu(void);
	void	ConnectNetObj(void);	
	void	ConnectNeuNet(void);
	void	ConnectNeuNeu(void);
	void	ConnectNeuObj(void);
	void	ConnectObjNet(void);
	void	ConnectObjNeu(void);
	void	ConnectObjObj(void);
	virtual Boolean	ConnectModels(CNModel **aFixedModel, CNModel **aDraggedModel);
};





