/******************************************************************************
 CNOVDraggerTask.c

	SUPERCLASS = CMouseTask 
	
	Copyright  1993 by Daniel McFarlane. All rights reserved.
	
 ******************************************************************************/

#include "CNOVDraggerTask.h"
#include "CNBrainPan.h"
#include "CNOViewOwner.h"
#include "CNHotSpotView.h"
#include "CNObjectView.h"
#include "CApplication.h"
#include "NervousSystemApp.h"
#include "NervousSystemDoc.h"
#include "CList.h"
#include "CNHotSpotView.h"
#include "CNOVConnectTask.h"

extern CApplication	*gApplication;
extern CNervousSystemDoc *gNLimbo;


/******************************************************************************
 INOVDraggerTask
 
******************************************************************************/

void CNOVDraggerTask::INOVDraggerTask(
	short 				aNameIndex, 
	CNObjectView		*aDraggableNOView,
	CNObjectView		*aGraspedNOView,
	CNBrainPan 			*aNBrainPan)
{
	CTask::ITask(aNameIndex);
	
	itsNBrainPan = aNBrainPan;
	itsNObjectView = aDraggableNOView;
	fDragged = FALSE;
	
	// Initialize "itsNOVOwner".
	itsNOVOwner = NULL;
	itsNOVOwner = itsNBrainPan->CopyItsSelection(gNLimbo);
	
	// Initialize "boundingRect".
	itsNOVOwner->GetBoundingRect(&boundingRect); 

	// Initialize "itsHSpotsNOVOwner".
	itsHSpotsNOVOwner = NULL;
	itsHSpotsNOVOwner = aGraspedNOView->CopyItsHotSpots(gNLimbo);
	
	matchedHSpotFixed = NULL;
	matchedHSpotDragged = NULL;
	aNOVConnectTask = NULL;
}


/******************************************************************************
 EndTracking

		Mouse down tracking has ended because the user has released the
		mouse button.
 ******************************************************************************/

void	CNOVDraggerTask::EndTracking(
	LongPt		*currPt,
	LongPt		*prevPt,
	LongPt		*startPt)
{
	// "Track" the last point. 
	KeepTracking(currPt, prevPt, startPt);
	
	// We are done dragging, so HotSpots become invisible again.
	itsNOVOwner->SetFBeingDraggedAll(FALSE);
	itsHSpotsNOVOwner->SetFDrawHotSpotsAll(FALSE);
	
	// Make a connection if needed.
	if ((matchedHSpotFixed) && (matchedHSpotDragged)) {
		
		matchedHSpotFixed->SetFDrawHotSpot(FALSE);

		// Note: what we really need here is a class that would manage a reference to a single
		// object instead of CNOViewOwner that manages a list of references to objects.
		// "matchedHSpotFixed" "matchedHSpotDragged" could be instances of this class, and
		// then we could pass them into "->INOVConnectTask(" without them being corrupted
		// when they the actual CNHotSpot objects are copied into a new graphical environment
		// as part of the CNOVConnectTask.
		// However, rather than write that new class I leave it as a future exercise for 
		// someone else.

		// Make the connection.
		aNOVConnectTask = new(CNOVConnectTask);
		aNOVConnectTask->INOVConnectTask(matchedHSpotFixed, matchedHSpotDragged, itsNBrainPan);
		aNOVConnectTask->Do();
	}
}


/******************************************************************************
 KeepTracking
 
 	Called continously while the mouse button is help down. This method
 	continously updates the selection and autoscrolls the table.
 	
 	A note on why we did not use the Macintosh ToolBox call "DragGrayGgn()"
 	to animate the dragging of itsNOVOwner: we need to Calculate the occurrences
 	of the HSpots of the object we grasp with the mouse (itsHSpotsNOVOwner), 
 	intersecting with the HSpots of the other objects that are not being dragged.  
 	With "DragGrayGgn()" you cannot do any kind of extra calculation durring the drag.
 	
******************************************************************************/

void CNOVDraggerTask::KeepTracking(
	LongPt		*currPt,
	LongPt		*prevPt,
	LongPt		*startPt)
{
	LongPt		delta;
	LongPt		prevOffset;
	LongRect	beforeRect;
	LongRect	afterRect;
	LongRect	combinedRectLong;
	Rect		combinedRect;
	LongRect	tempLongRect;
	Rect		tempRect;
	
	if ((itsNBrainPan->AutoScroll(currPt)) || (!(EqualLongPt(currPt, prevPt))) ) {
		
		// Initialize matchedHSpotFixed
		if (matchedHSpotFixed) {
			matchedHSpotFixed->SetFDrawHotSpot(FALSE);
			
			// ReDraw the HotSpotView's area.
			matchedHSpotFixed->GetFrame(&tempLongRect);
			matchedHSpotFixed->FrameToItsNBrainPanR(&tempLongRect);
			LongToQDRect(&tempLongRect, &tempRect);
			itsNBrainPan->Draw(&tempRect);
		}
		matchedHSpotFixed = NULL;
		
		// Initialize matchedHSpotDragged.
		matchedHSpotDragged = NULL;
		
		// Get bounding Rect of itsNOVOwner before the move.
		prevOffset = *prevPt;
		SubLongPt(startPt, &prevOffset);
		beforeRect = boundingRect;
		OffsetLongRect(&beforeRect, prevOffset.h, prevOffset.v);
		
		// Calculate the delta.
		delta = *currPt;
		SubLongPt(prevPt, &delta);
		
		// Drag itsNOVOwner to its new location.
		itsNOVOwner->MoveByAll(&delta);
		
		// Get bounding Rect of itsNOVOwner after the move.
		afterRect = beforeRect;
		OffsetLongRect(&afterRect, delta.h, delta.v);
		
		// See if one of the hot spots of the object that I grasp is intersecting with
		// someone else's hot spot.  
		// "matchedHSpotFixed" and "matchedHSpotDragged" are assigned here.
		GetPossibleConnection(itsHSpotsNOVOwner);
		if (matchedHSpotFixed)
			matchedHSpotFixed->SetFDrawHotSpot(TRUE);
			
		// INV: if there exists a matching pair then "matchedHSpotFixed" and 
		// "matchedHSpotDragged" are now assigned to it.
		
		// Do housekeeping on itsNBrainPan::myDataExtent.
		itsNBrainPan->CalcMyDataExtent();
		
		// ReDraw the affected area of itsNBrainPan.
		UnionLongRect(&beforeRect, &afterRect, &combinedRectLong);
		LongToQDRect(&combinedRectLong, &combinedRect);
		itsNBrainPan->Draw(&combinedRect);
		
		// The itsNOVOwner was actually dragged -- this is not just a selection click.
		fDragged = TRUE;
	}
}


/******************************************************************************
 GetPossibleConnection 
	
	Determine whether any of itsNBrainPan->itsSubview's NObjectViews
	have NHSpots intersecting with the NHSpots of the object being dragged.
	This method returns the first matching pair of NHSpots.
	We narrow our search for matching HSpots by restricting our search to objects
	that match the following criteria: the object is enclosed only by the CNBrainPan
	(here we assume that all of these "top-level-object" have maintained a list of 
	all the HSpots they enclose and that their subviews enclose); and the object is 
	ReallyVisible; and the object is not part of itsNBrainPan->itsSelection (and therefore 
	not being currently dragged by "this".
	
	The result of calling this method (which supsequently calls the method 
	"CoompareHSpots") is to test for matching NHSpots between what is being dragged
	and what is not and to assign the first matching pair to the ivars matchedHSpotFixed and
	matchedHSpotDragged.
 ******************************************************************************/

void CNOVDraggerTask::GetPossibleConnection(CNOViewOwner *graspedHSpotsNOVOwner)
{
	CList			*tempNBPanSubviews = NULL;
	long			numItems;
	long			i;
	CNObjectView	*nextSubview = NULL;
	CNOViewOwner	*subviewHSpotsNOVOwner = NULL;
	
	// Initialize a working copy of itsNBrainPan->itsSusbviews.
	tempNBPanSubviews = itsNBrainPan->GetSubviews();
	
	if (tempNBPanSubviews) {
		
		numItems = tempNBPanSubviews->GetNumItems();
		for (i = 1; i <= numItems; ++i) {
		
			// Access the next item in the copy of itsNBrainPan->itsSubview's list.
			nextSubview = (CNObjectView *) tempNBPanSubviews->NthItem(i);
			
			// Only consider testing for a match if nextSubview is really visible.
			if (nextSubview->ReallyVisible()) {
			
				// Only consider testing for a match if nextSubview is not part of 
				// itsNBrainPan->itsSelection.
				if (!(nextSubview->IsSelected())) {
				
					subviewHSpotsNOVOwner = nextSubview->CopyItsHotSpots(itsNBrainPan);

					CompareHSpots(graspedHSpotsNOVOwner, subviewHSpotsNOVOwner);

					if (subviewHSpotsNOVOwner) {
						subviewHSpotsNOVOwner->Dispose();
						subviewHSpotsNOVOwner = NULL;
					}
						
					if ((matchedHSpotFixed) && (matchedHSpotDragged))  // a matching pair
						break;
				}
			}
		}
	
		tempNBPanSubviews->Dispose();
	}
}


/******************************************************************************
 CompareHSpots 
	
	Assign "matchedHSpotFixed" and "matchedHSpotDragged" if a match exists.
 ******************************************************************************/

void CNOVDraggerTask::CompareHSpots(CNOViewOwner *graspedHSpotsNOVOwner, 
			CNOViewOwner *subviewHSpotsNOVOwner)
{
	CNHotSpotView	*graspedNHotSpotView = NULL;
	CNHotSpotView	*subviewNHSpotView = NULL;
	long	i, j;
	long	numItemsHeld, numItemsSubview;
	LongRect	graspedBoundLongRect;
	Rect		graspedBoundRect;
	LongRect	subviewBoundLongRect;
	Rect		subviewBoundRect;
	Boolean		fMatched = FALSE;
	CList		*graspedList = NULL;
	CList		*subviewList = NULL;
	
	// Are there really two lists to compare?
	if ((graspedHSpotsNOVOwner) && (subviewHSpotsNOVOwner)) {
		
		// Initialize working copies of the owned lists and how many objects in each.
		graspedList = graspedHSpotsNOVOwner->GetPossessions();
		subviewList = subviewHSpotsNOVOwner->GetPossessions();
		numItemsHeld = graspedList->GetNumItems();
		numItemsSubview = subviewList->GetNumItems();
		
		// Look at each of the grasped's objects.
		for (i = 1; i <= numItemsHeld; ++i) {
		
			graspedNHotSpotView = (CNHotSpotView *) graspedList->NthItem(i);
			graspedNHotSpotView->GetFrame(&graspedBoundLongRect);
			graspedNHotSpotView->FrameToWindR(&graspedBoundLongRect, &graspedBoundRect);
			
			// Look at each of the subview's objects.
			for (j = 1; j <= numItemsSubview; ++j) {
		
				subviewNHSpotView = (CNHotSpotView *) subviewList->NthItem(j);
				subviewNHSpotView->GetFrame(&subviewBoundLongRect);
				subviewNHSpotView->FrameToWindR(&subviewBoundLongRect, &subviewBoundRect);				
				
				// See if the bounding rectangles of these two hotSpots intersect.
				if (SectRect(&graspedBoundRect, &subviewBoundRect, &subviewBoundRect)) {
					matchedHSpotFixed = subviewNHSpotView;
					matchedHSpotDragged = graspedNHotSpotView;
					fMatched = TRUE;
					break;
				}
			}
			
			if (fMatched)
				break;
		}
		
		if (graspedList) 
			graspedList->Dispose();
		if (subviewList) 
			subviewList->Dispose();
	}
}


/******************************************************************************
 BeginTracking
 
 	Called at the start of mouse tracking. This method calculates the
 	anchor cell, and makes the initial selection based upon the
 	current selection settings (i.e. the selection flags, modifier keys,
 	and the previously initialized instance variables that control selection).
 	
******************************************************************************/

void CNOVDraggerTask::BeginTracking( LongPt *startPt)
{	
	itsNOVOwner->SetFBeingDraggedAll(TRUE);
	itsNOVOwner->InitGraphicalBookmarkAll();
	
	itsHSpotsNOVOwner->SetFDrawHotSpotsAll(TRUE);
	
}


/******************************************************************************
 Undo 

 ******************************************************************************/

void	CNOVDraggerTask::Undo()
{
	// Undo the connection.
	if (aNOVConnectTask) {
		aNOVConnectTask->Undo();
		
		// Reset the values of the ivars that point to the objects that are being
		// disconnected.  Because the objects have been transported to and from another
		// graphical environment, the original pointers are bogus.
		matchedHSpotDragged = aNOVConnectTask->GetDraggedHSpot();
		matchedHSpotFixed = aNOVConnectTask->GetFixedHSpot();
		itsNObjectView = (CNObjectView *) matchedHSpotDragged->itsEnclosure;		
	}
	
	// Undo the drag.
	itsNOVOwner->ToggleMoveToGBookmarkAll();
	
	// Do housekeeping on itsNBrainPan::myDataExtent.
	itsNBrainPan->CalcMyDataExtent();

	inherited::Undo();
}


/******************************************************************************
 Redo

 ******************************************************************************/

void	CNOVDraggerTask::Redo()
{
	itsNOVOwner->ToggleMoveToGBookmarkAll();
	
	// Redo the connection.
	if (aNOVConnectTask) {
		aNOVConnectTask->Redo();
		
		// Reset the values of the ivars that point to the objects that are being
		// disconnected.  Because the objects have been transported to and from another
		// graphical environment, the original pointers are bogus.
		matchedHSpotDragged = aNOVConnectTask->GetDraggedHSpot();
		matchedHSpotFixed = aNOVConnectTask->GetFixedHSpot();
		itsNObjectView = (CNObjectView *) matchedHSpotDragged->itsEnclosure;		
	}

	// Do housekeeping on itsNBrainPan::myDataExtent.
	itsNBrainPan->CalcMyDataExtent();

	inherited::Undo();
}


/******************************************************************************
 Dispose
  	
  	Dispose of a CNOVDraggerTask. This method clears the table's reference to
  	this task.
******************************************************************************/

void CNOVDraggerTask::Dispose(void)
{
	if (aNOVConnectTask)
		aNOVConnectTask->Dispose();
	
	if (itsNOVOwner) 
		itsNOVOwner->Dispose();
	
	if (itsHSpotsNOVOwner) 
		itsHSpotsNOVOwner->Dispose();

	inherited::Dispose();				/* Pass message on to superclass	*/
}


/******************************************************************************
 DidYouDrag

 ******************************************************************************/

Boolean	CNOVDraggerTask::DidYouDrag(void)
{
	return fDragged;
}


