/******************************************************************************
 CNOVDraggerTask.h

		Interface for CNOVDraggerTask class
		
	SUPERCLASS = CMouseTask
	
	Copyright  1993 by Daniel McFarlane. All rights reserved.
	
	Instances of this class manage dragging selected groups of instances of 
	CNObjectView and possibly connecting them to non-dragged objects.
	
	For a selected group of instances of CNObjectView that is being dragged
	(refered to with an instance of CNOViewOwner), the one object that has
	the mouse pointer on it is "hot".  The one "hot" object that is being
	dragged has its CNHotSpots visible.  When those CNHotSpots intersect
	the CNHotSpots of a non-dragging object, then those non-dragging object's 
	CNHotSpots are also highlighted.  This indicates that there is a potential 
	connection between the "hot" dragged object and that specific non-dragging
	object.  If the user releases the mouse button at a place with a potential
	connection, then "this" manages the execution of a CNOVConnectionTask. 
 ******************************************************************************/


#pragma once

#include "CMouseTask.h"
#include "LongCoordinates.h"

class CNBrainPan;
class CNOViewOwner;
class CNObjectView;
class CNHotSpotView;
class CList;
class CNOVConnectTask;


class CNOVDraggerTask : public CMouseTask
{
public:

	void	INOVDraggerTask(short aNameIndex, CNObjectView *aDraggableNOView,
					CNObjectView *aGraspedNOView, CNBrainPan *aNBrainPan);
	
	void	Dispose(void);
	void	BeginTracking( LongPt *startPt);
	void	KeepTracking( LongPt *currPt, LongPt *prevPt, LongPt *startPt);
	void	EndTracking(struct LongPt *currPt, struct LongPt *prevPt, struct LongPt *startPt);
	Boolean	DidYouDrag(void);
	void	Undo(void);
	void	Redo(void);
	void	GetPossibleConnection(CNOViewOwner *heldHSpotsNOVOwner);
	void	CompareHSpots(CNOViewOwner *heldHSpotsNOVOwner, 
					CNOViewOwner *subviewHSpotsNOVOwner);
	
protected:
	CNOVConnectTask	*aNOVConnectTask;
	CNBrainPan		*itsNBrainPan;  // The current workspace where the dragging is happening.
	CNObjectView	*itsNObjectView;  // The grasped NObjectView of the selection being dragged. 
	CNOViewOwner	*itsNOVOwner;  // The selection being dragged.
	Boolean			fDragged;
	CNOViewOwner 	*itsHSpotsNOVOwner;  // A copy of the HSpots of itsNObjectView.
	CNHotSpotView	*matchedHSpotFixed;
	CNHotSpotView	*matchedHSpotDragged;
	LongRect		boundingRect;
};