/******************************************************************************
 CNOViewCmdTask.c

							The NOViewCmdTask Class
		
	SUPERCLASS = CTask
	
	Copyright  1993 by Daniel McFarlane. All rights reserved.

 ******************************************************************************/

#include "CNOViewCmdTask.h"
#include "CNNervousClipboard.h"
#include "CNOViewOwner.h"
#include "NervousSystemApp.h"
#include "NervousSystemDoc.h"
#include "CNBrainPan.h"

// External declarations from "GlobalVars.c"
extern CClipboard	*gClipboard;
extern CApplication	*gApplication;
extern CNervousSystemDoc *gNLimbo;


/******************************************************************************
 INOViewCmdTask

		Initialize a NOViewCmdTask
 ******************************************************************************/

void	CNOViewCmdTask::INOViewCmdTask(
	short		aNameIndex,
	long 		theCommand,
	CNBrainPan	*aNBrainPan)
{
	CTask::ITask(aNameIndex);
	
	itsNBrainPan = aNBrainPan;
	myCmd = theCommand;
	itsNOVOwnerFromClip = NULL;

	// Initialize "fDistructiveCommand"
	switch (myCmd) {
		case cmdCut:
			fDistructiveCommand = TRUE;
			break;
			
		case cmdCopy:
			fDistructiveCommand = FALSE;
			break;
			
		case cmdPaste:	
			// Initialize itsNOVOwnerFromClip as an actual copy of what's in the clipboard.
			((CNNervousClipboard *) gClipboard)->GetPrivateScrap('NSEL', &itsNOVOwnerFromClip);
			
			fDistructiveCommand = TRUE;
			break;
			
		case cmdClear:
			fDistructiveCommand = TRUE;
			break;
			
		case cmdSelectAll:
			fDistructiveCommand = FALSE;
			break;
			
		default:
			printf("Never heard of that command, {CNOViewCmdTask::INOViewCmdTask}\n");
			exit(1);
			break;
	}	
	
	// Initialize "itsNOVOwner" array.
	itsNOVOwner = NULL;
	itsNOVOwner = itsNBrainPan->CopyItsSelection(gNLimbo);
}


/******************************************************************************
 Do

		Perform a NOViewCmdTask
 ******************************************************************************/

void	CNOViewCmdTask::Do()
{
	CNOViewOwner *tempNOVOwner = NULL;
	
	switch (myCmd) {
	
		case cmdCut:
			
			// Remove the selection from intNBrainPan to Limbo.
			itsNBrainPan->RemoveNOViews(itsNOVOwner);

			// Create an actual copy of itsNOVOwner and send it to the gClipboard.
			tempNOVOwner = itsNOVOwner->CopyNOVOwnerActualFor(gNLimbo->GetItsMainPane());
			((CNNervousClipboard *) gClipboard)->EmptyPrivateScrap();
			((CNNervousClipboard *) gClipboard)->PutPrivateScrap('NSEL', tempNOVOwner);
			tempNOVOwner->Dispose();
			break;
		
		// Copy is Cut without Removing.
		case cmdCopy:
		
			// Create an actual copy of itsNOVOwner and send it to the gClipboard.
			tempNOVOwner = itsNOVOwner->CopyNOVOwnerActualFor(gNLimbo->GetItsMainPane());
			((CNNervousClipboard *) gClipboard)->EmptyPrivateScrap();
			((CNNervousClipboard *) gClipboard)->PutPrivateScrap('NSEL', tempNOVOwner);
			tempNOVOwner->Dispose();
			break;
			
		case cmdPaste:	
			// Replace whatever was currently selected.
			if (itsNOVOwner)
				itsNBrainPan->RemoveNOViews(itsNOVOwner);
			
 			// Paste itsNOVOwnerFromClip into itsNBrainPan.
			if (itsNOVOwnerFromClip) {
			
				// Make a shadow copy of itsNOVOwnerFromClip to send to itsNBrainPan; 
				tempNOVOwner = itsNOVOwnerFromClip->CopyNOVOwnerFor(gNLimbo);
				itsNBrainPan->AddNOVObjects(tempNOVOwner);
				tempNOVOwner->Dispose();
			}
			break;
 		
 		// Clear is Cut without saving a copy to the clipboard.
		case cmdClear:
			itsNBrainPan->RemoveNOViews(itsNOVOwner);
			break;
			
		case cmdSelectAll:
			break;

		default:
			printf("Never heard of that command, {CNOViewCmdTask::Do}\n");
			exit(1);
			break;
	}

	tempNOVOwner = NULL;
}


/******************************************************************************
 Undo

		Undo a task. Subclasses must override this method.
 ******************************************************************************/

void	CNOViewCmdTask::Undo()
{
	CNOViewOwner *tempNOVOwner = NULL;

	switch (myCmd) {
	
		case cmdCut:
			// Make a shadow copy of itsNOVOwner to send to itsNBrainPan.
			tempNOVOwner = itsNOVOwner->CopyNOVOwnerFor(gNLimbo);
			itsNBrainPan->AddNOVObjects(tempNOVOwner);
			tempNOVOwner->Dispose();
			break;
			
		case cmdCopy:				
			break;
			
		case cmdPaste:
			// Get back the NOVOwner that was put into itsNBrainPan from the clipboard.
			if (itsNOVOwnerFromClip) {
				itsNBrainPan->RemoveNOViews(itsNOVOwnerFromClip);
			}
			
			// Put back the original selection that was replaced with the Paste.
			if (itsNOVOwner) {
				// Make a shadow copy of itsNOVOwner to send to itsNBrainPan.
				tempNOVOwner = itsNOVOwner->CopyNOVOwnerFor(gNLimbo);
				itsNBrainPan->AddNOVObjects(tempNOVOwner);
				tempNOVOwner->Dispose();
			}
 			break;
 			
		case cmdClear:
			// Make a shadow copy of itsNOVOwner to send to itsNBrainPan.
			tempNOVOwner = itsNOVOwner->CopyNOVOwnerFor(gNLimbo);
			itsNBrainPan->AddNOVObjects(tempNOVOwner);
			tempNOVOwner->Dispose();
			break;
		
		case cmdSelectAll:
			break;

		default:
			printf("Never heard of that command, {CNOViewCmdTask::Do}\n");
			exit(1);
			break;
	}
	
	inherited::Undo();
}


/******************************************************************************
 Redo

		Redo a task which has been undone. This method assumes that Redo
		is the same as Undo. Subclasses which implement Redo differently
		from Undo must override this method.
 ******************************************************************************/

void	CNOViewCmdTask::Redo()
{
	CNOViewOwner *tempNOVOwner = NULL;

	switch (myCmd) {
	
		case cmdCut:
			itsNBrainPan->RemoveNOViews(itsNOVOwner);
			break;
			
		case cmdCopy:				
			break;

		case cmdPaste:				
 			// Re-Remove whatever was selected to begin with.
			if (itsNOVOwner)
				itsNBrainPan->RemoveNOViews(itsNOVOwner);
			
 			// Re-Paste itsNOVOwnerFromClip into itsNBrainPan.
			if (itsNOVOwnerFromClip) {
			
				// Make a shadow copy of itsNOVOwnerFromClip to send to itsNBrainPan; 
				tempNOVOwner = itsNOVOwnerFromClip->CopyNOVOwnerFor(gNLimbo);
				itsNBrainPan->AddNOVObjects(tempNOVOwner);
				tempNOVOwner->Dispose();
			}
			break;
		
		case cmdClear:
			itsNBrainPan->RemoveNOViews(itsNOVOwner);
			break;
			
		case cmdSelectAll:
			break;

		default:
			printf("Never heard of that command, {CNOViewCmdTask::Redo}\n");
			exit(1);
			break;
	}

	inherited::Undo();
}


/******************************************************************************
 Dispose     "Dispose of a CNOViewCmdTask object"
 
 ******************************************************************************/

void	CNOViewCmdTask::Dispose()
{
	// If the new object that was created here, "itsNewNOView" was Undone, but 
	// not Redone, then it is in limbo and needs to be disposed of with this task.
	if (itsNOVOwner) {
		if ((!undone) && (fDistructiveCommand)) 
			itsNOVOwner->DisposeAll();
		else
			itsNOVOwner->Dispose();
	}
	
	// If something was Pasted into itsNBrainPan from here and not undone then just
	// Dispose of the NOVOwner pointing to those objects.  However, if the Paste was
	// undone then Dispose of the objects too.
	if (itsNOVOwnerFromClip) {
		if (!undone)
			itsNOVOwnerFromClip->Dispose();
		else
			itsNOVOwnerFromClip->DisposeAll();
	}
	
	inherited::Dispose();				/* Pass message on to superclass	*/
}


