/******************************************************************************
 CNSomaView.c

	The NSomaView Class
		
	SUPERCLASS = CNObjectView
	
	Source code for "NervousSystem"
	Copyright  1994 Daniel McFarlane. All rights reserved.
	All use is dictated by the terms of the "GNU General Public 
	License" (see the file "CNObjectView.c" for details).
 ******************************************************************************/

#include "Global.h"
#include "CNSomaView.h"
#include <stdio.h>
#include "LongQD.h"
#include "CNOViewTask.h"
#include "MainWindow.h"
#include "SomaController.h"
#include "CNSomaModel.h"
#include "NervousSystemDoc.h"



/******************************************************************************
 INSomaView

		Initialize a NSomaView object
 ******************************************************************************/

void	CNSomaView::INSomaView(
	CView			*anEnclosure,
	CBureaucrat		*aSupervisor,
	short			aWidth,
	short			aHeight,
	short			aHEncl,
	short			aVEncl,
	SizingOption	aHSizing,
	SizingOption	aVSizing,
	long 			aHFrameOrigin, 
	long 			aVFrameOrigin, 
	CNSomaModel		*aNSomaModel)
{
	CNObjectView::INObjectView(anEnclosure, aSupervisor,
					aWidth, aHeight, aHEncl, aVEncl, aHSizing, aVSizing, 
					aHFrameOrigin, aVFrameOrigin);
	
	// Initialize the instance of the NModelElem managed by this view.
	if (aNSomaModel)
		myNModelElem = aNSomaModel->CopyModel();
	else {
		myNModelElem = new(CNSomaModel);
		((CNSomaModel *) myNModelElem)->INSomaModel();
	}
	
	wantsClicks = TRUE;
	SetCanBeGopher(TRUE);
}


/******************************************************************************
 CopyNOViewInto {OVERRIDE}
 
******************************************************************************/

CNObjectView *CNSomaView::CopyNOViewInto(CView *anEnclosure, CBureaucrat *aSupervisor)
{
	CNSomaView		*theCopy = NULL;
	
	// Create a new CNObjectView as a copy.
	theCopy = new(CNSomaView);

	// Initialize the new object to be a copy of "this".
	theCopy->INSomaView(anEnclosure, aSupervisor, width, height, 
				hEncl, vEncl, hSizing, vSizing, hFrameOrigin, vFrameOrigin, 
				((CNSomaModel *) myNModelElem));
	
	INOViewCopy(theCopy);
	
	return theCopy;
}


/******************************************************************************
 INOViewCopy 
 
 	Initialize a copy of this NOView.
******************************************************************************/

void CNSomaView::INOViewCopy(CNObjectView *theCopy)
{
	((CNSomaView *) theCopy)->itsGraphicData = itsGraphicData;
	
	inherited::INOViewCopy(theCopy);
}


/******************************************************************************
 DoClick {OVERRIDE}

		Mouse click inside the NBrainPan
 ******************************************************************************/

void	CNSomaView::DoClick(
	Point		hitPt,
	short		modifierKeys,
	long		when)
{
	inherited::DoClick(hitPt, modifierKeys, when);
}
	

/******************************************************************************
 Dispose 

		Dispose of a NSomaView
 ******************************************************************************/

void	CNSomaView::Dispose()
{
		/* makes sure that the last undo task is disposed. Since the 	*/
		/* undo tasks usually have references back to this pane, 		*/
		/* it would be bad if this pane was disposed and the task was 	*/
		/* not informed.												*/

	inherited::Dispose();			/* Pass message on to superclass	*/
}


/******************************************************************************
 DoDialog
 
 ******************************************************************************/

void	CNSomaView::DoDialog(void)
{
	CNervousSystemDoc	*aNervousSystemDoc = NULL;
	
	aNervousSystemDoc = GetMyNervousSysDoc();
	DoSomaController(aNervousSystemDoc, ((CNSomaModel *) myNModelElem));
}









