/******************************************************************************
 CNSynapseView.c

	The NSynapseView Class
		
	SUPERCLASS = CNObjectView
	
	Source code for "NervousSystem"
	Copyright  1994 Daniel McFarlane. All rights reserved.
	All use is dictated by the terms of the "GNU General Public 
	License" (see the file "CNObjectView.c" for details).
 ******************************************************************************/

#include "Global.h"
#include "CNSynapseView.h"
#include <stdio.h>
#include "LongQD.h"
#include "CNOViewTask.h"
#include "MainWindow.h"
#include "CNSynapseModel.h"
#include "SynapseController.h"
#include "NervousSystemDoc.h"



/******************************************************************************
 INSynapseView

		Initialize a NSynapseView object
 ******************************************************************************/

void	CNSynapseView::INSynapseView(
	CView			*anEnclosure,
	CBureaucrat		*aSupervisor,
	short			aWidth,
	short			aHeight,
	short			aHEncl,
	short			aVEncl,
	SizingOption	aHSizing,
	SizingOption	aVSizing,
	long 			aHFrameOrigin, 
	long 			aVFrameOrigin, 
	CNSynapseModel		*aNSynapseModel)
{

	CNObjectView::INObjectView(anEnclosure, aSupervisor,
					aWidth, aHeight, aHEncl, aVEncl, aHSizing, aVSizing, 
					aHFrameOrigin, aVFrameOrigin);
					
	// Initialize the instance of the NModelElem managed by this view.
	if (aNSynapseModel)
		myNModelElem = aNSynapseModel->CopyModel();
	else {
		myNModelElem = new(CNSynapseModel);
		((CNSynapseModel *) myNModelElem)->INSynapseModel();
	}
	
	wantsClicks = TRUE;
	SetCanBeGopher(TRUE);
}


/******************************************************************************
 DoDialog  

 ******************************************************************************/

void	CNSynapseView::DoDialog(void)
{
	CNervousSystemDoc	*aNervousSystemDoc = NULL;
	
	aNervousSystemDoc = GetMyNervousSysDoc();
	DoSynapseController(aNervousSystemDoc, ((CNSynapseModel *) myNModelElem));
}



/******************************************************************************
 CopyNOViewInto {OVERRIDE}
 
******************************************************************************/

CNObjectView *CNSynapseView::CopyNOViewInto(CView *anEnclosure, CBureaucrat *aSupervisor)
{
	CNSynapseView		*theCopy = NULL;
	
	// Create a new CNObjectView as a copy.
	theCopy = new(CNSynapseView);

	// Initialize the new object to be a copy of "this".
	theCopy->INSynapseView(anEnclosure, aSupervisor, width, height, 
				hEncl, vEncl, hSizing, vSizing, hFrameOrigin, vFrameOrigin, 
				((CNSynapseModel *) myNModelElem));
	
	INOViewCopy(theCopy);
	
	return theCopy;
}


/******************************************************************************
 INOViewCopy 
 
 	Initialize a copy of this NOView.
******************************************************************************/

void CNSynapseView::INOViewCopy(CNObjectView *theCopy)
{
	((CNSynapseView *) theCopy)->itsGraphicData = itsGraphicData;
	
	inherited::INOViewCopy(theCopy);
}


/******************************************************************************
 DoClick {OVERRIDE}

		Mouse click inside the NBrainPan
 ******************************************************************************/

void	CNSynapseView::DoClick(
	Point		hitPt,
	short		modifierKeys,
	long		when)
{
	inherited::DoClick(hitPt, modifierKeys, when);
}
	

/******************************************************************************
 Dispose 

		Dispose of a NSynapseView
 ******************************************************************************/

void	CNSynapseView::Dispose()
{
		/* makes sure that the last undo task is disposed. Since the 	*/
		/* undo tasks usually have references back to this pane, 		*/
		/* it would be bad if this pane was disposed and the task was 	*/
		/* not informed.												*/


	inherited::Dispose();			/* Pass message on to superclass	*/
}









