#pragma once

/******************************************************************************
 CNAxonModel.h

		Interface for the NAxon Class
		
		Source code for "NervousSystem"
		Copyright  1994 Joe Geigel. All rights reserved.
		All use is dictated by the terms of the "GNU General Public 
		License" (see the file "CNObjectView.c" for details).
		
	 	Biological Neural Net Simulator based on the Bon thesis
	 
	 	Object: Axon
	 
	 	An axon branch as defined by the Bon model
	 	Note that this is a subclass of CNModelElem.
 
 ******************************************************************************/
 
#define _H_CNAxonModel

class CNAxonView;
class CNDelayQueue;

#include "CNModelElem.h"	

class CNAxonModel : public CNModelElem {
private:
 	float			length;			/* length */
 	float 			velocity;		/* speed of signal propogation */
 	float			decay_coeff;	/* delay coefficient 
 										(should be 1 for active propagation */
 	CNDelayQueue	*future_pot;	/* queue containing future potentials */
  	static int 		count;			/* used to automatically name new Axons */

public:
	// Initialize and copy functions
	CNModel *CopyModel (void);
	int INAxonModel ( void );
	
	void GetAxonData (nAxonData *data);
	void SetAxonData (nAxonData *data);
 	
 	// simulation functions
 	void describe_self (FILE *f);
  	void process (void);
 	int prep_for_simulation (void);
};

 
 