/********************************************************************************
 CNModelElem.c
 
	Source code for "NervousSystem"
	Copyright  1994 Joe Geigel. All rights reserved.
	All use is dictated by the terms of the "GNU General Public 
	License" (see the file "CNObjectView.c" for details).

 	Biological Neural Net Simulator based on the Bon thesis
 
 
 	methods for Object: CNCompositeModel
 
 ********************************************************************************/
 
#include "CNCompositeModel.h"
#include "CNModelElem.h"
#include "CNLists.h"

// Prototypes for local functions
void DoConnctCopy (CObject *theObject);
void DoAddAndRegister (CObject *theObject, long param);
void DoAddNeuron (CObject *theObject, long param);
void DoAddConnection (CObject *theObject, long param);
void DoConnectionCopy (CObject *theObject, long param);

/**********************************************************************************
 *
 *                              CNCompositeModel
 *
 **********************************************************************************/
 
int CNCompositeModel::is_composite (void) { return 1; }

void CNCompositeModel::MakeConnections (CList *items) { }


/***********************************************************************************
 *
 *                              CNNeuronModel
 *
 ***********************************************************************************/

 
 /*
  *		init - Initialization 
  *
  */
  void CNNeuronModel::INNeuronModel (void)
  {
  		inherited::INModel();
  
  		us = new CNModelList;
  		us->Init ();		
  }
  
  
  void CNNeuronModel::add_model (CNModel *M)
  {
  		us->add (M);
  }
 
 
 /*
  *		DoAddAndRegister - Add an object to the list passed as a parameter and
  *			have the element register itself as a copy.  theObject is assumed to 
  *			be a CNModel.
  */
 void DoAddAndRegister (CObject *theObject, long param)
 {
 
 	CNModel *M = NULL;
 	CNModelList *L = NULL;
 	
 	M = (CNModel *)(theObject);
 	L = (CNModelList *)(param);
 	 	
 	L->add(M);
  	M->RegisterAsCopied ();
  }
  
  
/*
 *		DoConnectCopy - Have the object, interpretted as a CNMode, make a conection based
 *			on what was copied from it.
 */
 void DoConnctCopy (CObject *theObject)
 {
// 	CNModel *M = NULL;
 	
 	((CNModel *)(theObject))->MakeConnectionCopy ();
 }
 

 
 /*
  *		MakeConnection - Clist passed as argument is a list of elements to
  *			be placed in the array.  Connections are made by having these
  *			new elements reference the elements that they are copies of.
  */
 void CNNeuronModel::MakeConnections (CList *items)
 {
 	
 /*
  *		Go through each item in the list and place it into your list of component
  *		elements.  As you do this, have each element register itself as a copy 
  *		of whatever element it was copied from.
  */
  	
  	
  	items->DoForEach1 (DoAddAndRegister, (long)(us));
  		
  
 /*
  *		Now that all of the elements have had themselves registered as a copy,
  *		have the originals perform the connections on their copied representation.
  */
 
 	us->DoForEach (DoConnctCopy);
 	
 }
 
/*
 *		copyModel - Placed here for consistency...not much to do for the copy.
 */

CNModel *CNNeuronModel::CopyModel (void)
{
	CNNeuronModel *M;
	M = new CNNeuronModel;
	M->INNeuronModel ();
	
	M->RegisterAsCopy (this);
	
	return M;
}


/*
 *		findCopiedElem - find the element in the neuron that is a copy of the element
 *			passed in.
 */
 CNModel *CNNeuronModel::findCopiedElem (CNModel *elem)
 {
 	return (us->findCopy (elem));
 }



/******************************************************************************************
 *
 *                                      CNNetworkModel
 *
 ******************************************************************************************/
 
 /*
  *		init - perform initialization
  */
 void CNNetworkModel::INNetworkModel (void)
 {
 
 	inherited::INModel();
 	
  	us = new CNNeuronList;
  	us->Init ();
  	
  	connections = new CNConnectionList;
  	connections->Init();
 }
 
 void CNNetworkModel::define_connection (CNModel *in, CNModel *out)
 {
  	connections->add (in, out);
 }
 
 
 void CNNetworkModel::add_neuron (CNNeuronModel *M) 
 {
 	us->add_neuron(M);
 }
  
 
void DoAddNeuron (CObject *theObject, long param)
{
	CNNeuronModel *N;
	CNNeuronList *L;
	
	N = (CNNeuronModel *)(theObject);
	L = (CNNeuronList *)(param);
	
	L->add_neuron (N);
}


void DoAddConnection (CObject *theObject, long param)
{
	CNConnection *N;
	CNConnectionList *L;
	
	N = (CNConnection *)(theObject);
	L = (CNConnectionList *)(param);
	
	L->add_connection (N);
}


 void CNNetworkModel::fold_in_network (CNNetworkModel *N)
{
 	N->us->DoForEach1 (DoAddNeuron, (long) us);
 	N->connections->DoForEach1 (DoAddConnection, (long) connections);

}
 
 
/*
 *		copyModel - Return a clone of yourself.
 */
 CNModel *CNNetworkModel::CopyModel (void)
 
{

	CNNetworkModel *N;
	N = new CNNetworkModel;
	N->INNetworkModel ();
	
	N->RegisterAsCopy (this);
	
	delete N->connections;
	N->connections = connections->CopyList();
	
	return N;
}



	

void DoConnectionCopy (CObject *theObject, long param)
{
	CNConnection *C;
	CNNeuronList *L;
	CNModel *M_in, *M_out;
	
	C = (CNConnection *)(theObject);
	L = (CNNeuronList *)(param);
	
	M_in = L->findCopyElem (C->in);
	M_out = L->findCopyElem (C->out);
	
	if ((M_in) && (M_out)) {
//		CNModelElem::connect (M_out, M_in);
		C->in = M_in;
		C->out = M_out;
	}
}
	


/*
 *		MakeConnections - Make Connections based on a set of elements.  These
 *			elements are assumed to be CNNeuronModels.
 */
 
void CNNetworkModel::MakeConnections (CList *items)
{
/*
 *		Add each of the neurons to your neuron list
 */
 		items->DoForEach1 (DoAddNeuron, (long)(us));
 		
 /*
  *		Do the connection based on your connection list.
  */
  		connections->DoForEach1 (DoConnectionCopy, (long)(us));
  }
  
 