#pragma once

/******************************************************************************
 CNDendriteModel.h

		Interface for the NDendriteModel Class
		
		Source code for "NervousSystem"
		Copyright  1994 Joe Geigel. All rights reserved.
		All use is dictated by the terms of the "GNU General Public 
		License" (see the file "CNObjectView.c" for details).

	 	Biological Neural Net Simulator based on the Bon thesis
	 
	 	Object: CNDendriteModel
	
	 	A dendrite branch as defined by the Bon model
	 	Note that this is a subclass of CNModelElem.
		
 ******************************************************************************/
 
#define _H_CNDendrite

#include "CNModelElem.h"				

class DendriteSetup;
class CNDelayQueue;

class CNDendriteModel : public CNModelElem {
private:
 	float			acc_in_pot;		/* accumulated input potential */
 	float			acc_in_decay;	/* accumulated input decay constant */
 	float			aid_per_tick;	/* accumulated input decay const PER TICK */
 	float			length;			/* length */
 	float 			velocity;		/* speed of signal propogation */
 	float			decay_coeff;	/* delay coefficient (should be 1 for active propagation */
 	CNDelayQueue	*future_pot;	/* queue containing future potentials */
 	static int		count;			/* used to automatically name new Dendrites */
 	
public:
	// Initialize and copy functions
	CNModel *CopyModel (void);
	void SetDendriteData (nDendriteData *data);
	void GetDendriteData (nDendriteData *data);
	int INDendriteModel ( void );
 	
 	// simulation functions
 	void describe_self (FILE *f);
  	void process (void);
 	int prep_for_simulation (void);
};

 
 
