/*
 *
 *	Biological Neural Net Simulator based on the Bon thesis
 *
 *	Object: InputCell
 *
 *	External signal generators.  There are two types, pulse and train
 */
 
#pragma once

#include "CNModel.h"
#include "CNModelData.h"

#define MAX_SIGNALS 100

extern float ms_per_tick;

 class CNModelElem;
 class CNInputList;
 
 class CNInputModel : public CNModel{
	CNModelElem		*output;
	float		cur_t;

	static int count;
	static CNInputList	*all_of_us;
 protected:
 	char		name[100];
 public:
 	CNInputModel (void);
 	~CNInputModel (void);

 	int init (void);
 	void CopyConnect (void);
	void connect (CNModelElem *E);
 
 	void fire (void);
 	virtual float fire_val (float time);
 	virtual void reset(void);
 	
  	/* CNInputModel class list routines */
  	static void fire_inputs ( void );
  	static void simulation_prep (void);

// 	static CNInputModel *first_cell (void);
// 	static CNInputModel *next_cell (void);
};
 
 
 
 class PulseInput : public CNInputModel {
 	float	period;		/* fire every period ms */
 	float	voltage;	/* amount of potential to fire */
 public:
 	int Init (float per, float v);
 	
 	float fire_val (float time);
 	
 	void SetPulseInputData (nPulseInputData *data);
	void GetPulseInputData (nPulseInputData *data);
	
	CNModel *CopyModel(void);

 };
 
 
 typedef struct fireval {
 	float		time;		/* time to fire */
 	float	voltage;	/* how much to fire */
 } FireVal;
 
 
 class CNTrainCellModel : public CNInputModel {
	int		n_signals;				/* number of fire instructions */
	int		cur_signal;				/* next instruction to fire */
	FireVal	signals[MAX_SIGNALS];	/* firing instructions */
	
	char	fname[100];					/* name of file holding pulse list */
public:
	int Init (void);
	int InitF (char *filename, int check);
	
	int add_pulse (float when, float how_much);
	float fire_val (float time);
	
	void SetTrainInputData (nTrainInputData *data);
	void GetTrainInputData (nTrainInputData *data);
	
	CNModel *CopyModel(void);
 	void reset(void);

};
 
 