/*
 *
 *	Biological Neural Net Simulator based on the Bon thesis
 *
 *	methods for Object: NodeList, ElementList, InputList
 *
 */
 
#include "CNLists.h"
#include "CNNodeModel.h"
#include "CNModelElem.h"
#include "CNInputModel.h"
#include "CNCompositeModel.h"

// Prototypes for local functions
Boolean SameObject (CObject *object, long param);
Boolean InOutMatch (CObject *theObject, long param);
Boolean IsCopyOf (CObject *theObject, long param);
void DoAddToConnectionList (CObject *theObject, long param);
void DoConnectCopy (CObject *theObject);

void CNConnection::dummy (void) {}



/*
 *		CNList functions
 */
 
 void CNList::CLInit ( void )
 {
 	inherited::IList ();
 }
 
 
 
/*************************************************************************************
 *
 *                   CNConnectionList
 *
 *************************************************************************************/
 
 void CNConnectionList::Init (void)
 {
 	inherited::CLInit ();
 }
 
 
 void CNConnectionList::add (CNModel *in, CNModel *out)
 {
 	CNConnection *C;
 	
 	C = new CNConnection;
 	C->in = in;
 	C->out = out;
 	
 	this->Append (C);
 }
 
 
 void CNConnectionList::add_connection (CNConnection *C)
 {
 	this->Append(C);
 }
 
 
 void DoAddToConnectionList (CObject *theObject, long param)
 {
 	CNConnection *C;
 	CNConnectionList *L;
	
	C = (CNConnection *)(theObject);
	L = (CNConnectionList *)(param);
	
	L->add (C->in, C->out);
} 	
 
 
 CNConnectionList *CNConnectionList::CopyList (void)
 {
 	CNConnectionList *L;
 	
 	
 	L = new CNConnectionList;
 	L->Init ();
 	
 	this->DoForEach1 (DoAddToConnectionList, (long) L);
 	
 	return L;
 }
 	


/*************************************************************************************
 *
 *                   CNModelList
 *
 *************************************************************************************/
 
 void CNModelList::Init (void)
 {
 	inherited::CLInit ();
 }
 
 
 void CNModelList::add (CNModel *M)
 {
 	this->Append (M);
 }



Boolean IsCopyOf (CObject *theObject, long param)
{
	CNModel *C;
	CNModel *D;
	
	C = (CNModel *)(theObject);
	D = (CNModel *)(param);
	
	return ((C->copy_of) == D);
}


CNModel *CNModelList::findCopy (CNModel *M)
{
	return ((CNModel *)(this->FirstSuccess1 (IsCopyOf, (long) M)));
}


/*************************************************************************************
 *
 *                   CNNeuronList
 *
 *************************************************************************************/

 void CNNeuronList::Init (void)
 {
 	inherited::CLInit ();
 }
 
 
 void CNNeuronList::add_neuron (CNNeuronModel *M)
 {
 	this->Append (M);
 }
 
 
 CNModel *CNNeuronList::findCopyElem (CNModel *M)
 {
 	int n;
 	CNNeuronModel *N;
 	CNModel *C;
 	int i;
 	
 	n = this->GetNumItems();
 	for (i=1; i<= n; i++) {
 		N =  (CNNeuronModel *)(this->NthItem(i));
 		C = N->findCopiedElem (M);
 		if (C) return C;
 	}
 	
 	return C;
 }
 





/**************************************************************************************
 *
 *                    NodeList functions
 *
 *************************************************************************************/
 
 /*
  *		Init - Initialize your list to hold at most n elements, however, starting 
  *			empty.
  */
  
  void CNNodeList::Init (int n)
  {  		
  		max =n;
  		elems = 0;
  		
  		inherited::CLInit ();
  }
  
  
  
/*
 *		add - Add a node to the list.  If the list is full, no addition gets
 *			done.
 */
 void CNNodeList::add (CNNodeModel *N)
 {
  		if (elems >= max) return;
  		
		Append (N);
		elems++;
 }
  
  
  
 
 Boolean SameObject (CObject *object, long param)
 {
 	return (object == ((CObject *)param));
 }
 

/*
 *		del - Delete a node from the list.  If the node is not there, no deletion
 *			gets done.
 */
void CNNodeList::del (CNNodeModel *N)
{
	CObject *E;
	
	if (E = FirstSuccess1 (SameObject, (long)N)) {
		Remove (E);
		elems--;
	}
}


typedef struct in_out {
	CNModelElem	*in;
	CNModelElem	*out;
} InOut;


Boolean InOutMatch (CObject *theObject, long param)
{
//	theObject is a CNNodelModel, param is a pointer to an InOutStructure
	
	CNNodeModel *N;
	InOut		*I;
	
	N = (CNNodeModel *)theObject;
	I = (InOut *)param;
	
	return ((N->get_input() == I->in) && (N->get_output() == I->out));
}

/*
 *		find - Find a node from the list with  given input and output elements.
 *			Return NULL if not found.
 */
 CNNodeModel *CNNodeList::find (CNModelElem *in, CNModelElem *out)
 {
 
 	InOut I;
 	
 	I.in = in;
 	I.out = out;
 	
 	return ((CNNodeModel *)(FirstSuccess1 (InOutMatch, (long)(&I))));
 
 }



void DoConnectCopy (CObject *theObject)
{
	CNNodeModel *N;
	
	N = (CNNodeModel *)(theObject);
	N->CopyConnect ();
}

void CNNodeList::ConnectCopy (void)
{
	this->DoForEach (DoConnectCopy);
}


Boolean	CNNodeList::room_for_more ( void ) { return (elems < max); }
  		
 
/**************************************************************************************
 *
 *                   CNElementList functions
 *
 *************************************************************************************/
 
 
 /*
  *		Init - Initialize your list to hold at most n elements, however, starting 
  *			empty.
  */
  
  void CNElementList::Init ()
  {  		
  	  inherited::CLInit ();
   }
  
  
  
/*
 *		add - Add a Element to the list.  If the list is full, no addition gets
 *			done.
 */
 void CNElementList::add (CNModelElem *E)
 {
 	Append (E);
  }
  
  
  
/*
 *		del - Delete a Element from the list.  If the Element is not there, no deletion
 *			gets done.
 */
void CNElementList::del (CNModelElem *E)
{
	Remove (E);
}


  
 /**************************************************************************************
 *
 *                   CNInputList functions
 *
 *************************************************************************************/
 
/*
  *		Init - Initialize your list to hold at most n elements, however, starting 
  *			empty.
  */
  
 void CNInputList::Init ()
  {
  	inherited::CLInit();  		
  }
  
  
  
/*
 *		add - Add a Input to the list.  If the list is full, no addition gets
 *			done.
 */
 void CNInputList::add (CNInputModel *I)
 {
 	Append (I);
 }
  
  
  
/*
 *		del - Delete a Input from the list.  If the Input is not there, no deletion
 *			gets done.
 */
void CNInputList::del (CNInputModel *I)
{
	Remove(I);
}


