/*
 *
 *	Biological Neural Net Simulator based on the Bon thesis
 *
 *	Object: NodeList, ElementList, InputList
 *
 *	Lists used to store simulation objects.  Probably should be generalized
 *	to a single list class someday.
 */
 
#pragma once

#include <CList.h>
#include <CObject.h>

class CNModelElem;
class CNNodeModel;
class CNInputModel;
class CNModel;
class CNNeuronModel;
class CNNetworkModel;

class CNList : public CList {
public:
	void CLInit (void);
};

class CNConnection : public CObject {
public:
	CNModel	*in;
	CNModel *out;
	
	void dummy (void);
};


class CNConnectionList : public CNList {
public:
	void Init (void);
	
	void add (CNModel *in, CNModel *out);
	void add_connection (CNConnection *C);
	CNConnectionList *CopyList(void);
};


class CNModelList : public CNList {
public:
	void Init (void);
	
	void add (CNModel *M);
	CNModel *findCopy (CNModel *M);
};


class CNNeuronList : public CNModelList {
public:
	void Init (void);
	
	void add_neuron (CNNeuronModel *M);
	CNModel *findCopyElem (CNModel *M);
};


class CNNodeList : public CNList {
 	int		max;          /* maximum number of members */
 	int		elems;		  /* current number of members */
public:
 	void Init (int n);
 	
 	void add (CNNodeModel *N);
 	void del (CNNodeModel *N);
 	CNNodeModel *find (CNModelElem *in, CNModelElem *out);
 	
 	void ConnectCopy (void);
 	
 	Boolean	room_for_more ( void );
};


class CNElementList : public CNList {
public:
 	void Init (void);
 	
 	void add (CNModelElem *E);
 	void del (CNModelElem *E);
};



class CNInputList : public CNList {
public:
 	void Init (void);
 	
 	void add (CNInputModel *I);
 	void del (CNInputModel *I);
};

