/********************************************************************************
 CNModelElem
 
 	Source code for "NervousSystem"
	Copyright  1994 Joe Geigel. All rights reserved.
	All use is dictated by the terms of the "GNU General Public 
	License" (see the file "CNObjectView.c" for details).
 	
 	Object: CNModelElem
 
 	An CNModelElem is a basic component that makes up a neuron.  It is
 	a superclass and has four subclasses under it: Axon, Dendrite, Soma,
 	and Synapse.
 
 	At this level, all an element has is a current input and output potential
 	All attributes involved in individual element processing are contained
 	within the definition of the individual element type.
 
 *********************************************************************************/
 
 #pragma once
 
 #define PI 3.14159
 
#define MAX_INPUTS 1000
#define MAX_OUTPUT 1000

#include <stdio.h>
#include "CNModel.h"
#include "CNModelData.h"

 class CNNodeList;
 class CNNodeModel;
 class CNReporter;
 class CNElementList;
 

 class CNModelElem : public CNModel {
private:
 	char		name[100];

	float		cur_in_pot;		/* current input potential */
  	float		cur_out_pot;	/* current output potential */
  	
  	CNNodeList	*inputs;   /* list of input connections */
  	CNNodeList	*outputs;  /* list of output potentials */
  	
	static CNElementList	*all_of_us;

protected:  	
 	int			report_id;			   /* identification number used for printing */
  	Boolean		metered;               /* flag indicating if element is metered */
public:
 	/* constructor and initialization */
 	CNModelElem (void);
 	~CNModelElem (void);

 	void init (char *nme, int maxin, int maxout); 
 
 	/* routines involving simulation */
 	virtual void process (void);
 	virtual int prep_for_simulation (void);
 	
 	/* routines involving i/o potentials */
 	void clear_input (void);
 	void increment_input (float v);
 	float get_input (void);
 	void set_output (float v);
 	float get_output (void);
 	
 	
 	/* routines involving connections with other elements */
 	int new_input_allowed (void);
 	int new_output_allowed (void);
 	int connect_as_input (CNModelElem *E, float weight);
 	void disconnect_input (CNModelElem *E);
 	void add_input (CNNodeModel *N);
 	void add_output (CNNodeModel *N);
 	void delete_input (CNNodeModel *N);
 	void delete_output(CNNodeModel *N);
 	static int connect (CNModel *out, CNModel *in);
 	void CopyConnect (void);
 	void remove_node (CNNodeModel *N);
 	
 	
 	/* routines involving metering and printing */
 	virtual void describe_self (FILE *f);
 	void attach_meter(void);
 	void detach_meter(void);
 	void set_meter (Boolean OnOff);
 	const char *get_name(void);
 	void set_name (char *n);
 	void set_id (int id);
 	
 	/* rotines involving copying */
 	void MakeIdentical (CNModel *M);

 	/* Element class list access routines */
	static void simulation_prep (CNReporter *R);
 	static void process_elements ( void );
};
 
 