/*
 *
 *	Biological Neural Net Simulator based on the Bon thesis
 *
 *	methods for Object: CNNodeModel
 *
 */
 
#include "CNNodeModel.h"
#include "CNModelElem.h"
#include "CNLists.h"

// Prototypes for local functions
void DoTransfer (CObject *theObject);


/**************************************************************************************
 *
 *                        CNNodeModel  functions
 *
 **************************************************************************************/
 
 
/*
 *		define the global CNNodeList all_of_us to 0. Don't worry, it'll be
 *		redefined later.
 */
 
 CNNodeList *CNNodeModel::all_of_us = 0;
 
 
/*
 *		constructor - Gotta add yourself to the global list.
 */
 CNNodeModel::CNNodeModel(void)
 {
 		if (!all_of_us) {
 			all_of_us = new CNNodeList;
 			all_of_us->Init (20000);
 		}
 		
 		all_of_us->add (this);
 }
 
 
/*
 *		destructor - Remove yourself from the global list.
 */
 CNNodeModel::~CNNodeModel(void)
 {
 		all_of_us->del (this);
 }

 
/*
 *		Init - initialize your input and output.
 */
 void CNNodeModel::Init (CNModelElem *in, float wght, CNModelElem *out)
 {
 		inherited::INModel();
 
 		input = in;
 		output = out;
 		in_weight = wght;
 }
 
 
/*
 *		disconnect - You've been commanded to disband from one of
 *			your members, disconnect the other.
 */
void CNNodeModel::disconnect (CNModelElem *E)
{
	if (input != E)
		input->remove_node (this);
	else
		output->remove_node (this);
}

 

/*
 *		transfer - Takes inputs to a CNNodeModel and transfers to the output.  Considering
 *			the weight.
 */
 
 void CNNodeModel::transfer (void)
 {
 		output->increment_input (input->get_output() * in_weight);
 }
 
 
 
/*
 *		get_input, get_output - Retruns pointers to your input and output
 *			CNModelElems.
 */
 CNModelElem *CNNodeModel::get_input(void) { return input; }
 CNModelElem *CNNodeModel::get_output(void) { return output; }
 
 
void CNNodeModel::CopyConnect (void)
{
	if ((output->copied_as) && (input->copied_as))
		CNModelElem::connect (output->copied_as, input->copied_as);
}

 
 
 
 /***************************************************************************************
  *      
  *                CNNodeModel functions that act on the global node list.
  *
  ***************************************************************************************/
 
void DoTransfer (CObject *theObject)
{
	((CNNodeModel *)theObject)->transfer();
}
  
  
/*
 *		do_node_transfer - Have all Nodes perform their transfer
 */
static void CNNodeModel::do_node_transfers ( void )
{
	if (all_of_us) all_of_us->DoForEach (DoTransfer);
}
 
  
static CNNodeModel *CNNodeModel::find_node (CNModelElem *in, CNModelElem *out) 
  	{ return (all_of_us) ? all_of_us->find (in, out) : 0; }
  	
//  static CNNodeModel *CNNodeModel::first_node (void) 
//  	{ return (all_of_us) ? all_of_us->first() : 0; }
  	
//  static CNNodeModel *CNNodeModel::next_node (void) 
//  	{ return (all_of_us) ? all_of_us->next() : 0; }
 