/*
 *
 *	Biological Neural Net Simulator based on the Bon thesis
 *
 *	methods for Object: CNReporter, CNReportedEntity
 *
 */
 
 
 #include <stdio.h>
 #include "CNReporter.h"
 #include "CNModelElem.h"
 
 #include "CNSimulator.h"
 
 
/************************************************************************************
 *
 *                              CNReportedEntity Functions
 *
 ************************************************************************************/
 
 /*
  *		Init - You attached the reported entity with an CNModelElem.
  */
  
 void CNReportedEntity::Init (CNModelElem *E)
 {
 		elem = E;
 //		signals = 0;
 }
 
 
 /*
  *		clear- Clears your signal array and prepares for a simulation of
  *				time time steps.
  */
  void CNReportedEntity::clear (int time)
  {
// 		if (signals) free (signals);
//  		signals = (Blip *) malloc (time * sizeof (Blip));
  }
  
  
  
/*
 *	 	record - Record input and output signals for a given time step.
 */
 void CNReportedEntity::record (float in, float out, int t)
 {
 		signals[t].in = in;
 		signals[t].out = out;
 }
 
 
 
/*
 *		get_in/get_out - Retrieves input and output signals for a given time
 *			step.
 */
 
 
 float CNReportedEntity::get_in (int t) { return signals[t].in; }  
 float CNReportedEntity::get_out (int t) { return signals[t].out; }
 
 const char *CNReportedEntity::get_name (void) {return elem->get_name();}
 
 void CNReportedEntity::write_self (FILE *f) { elem->describe_self (f); }
 
 
 
/************************************************************************************'
 *
 *                      CNReporter functions
 *
 ************************************************************************************/
 
/*
 *		Constructor
 */
 CNReporter::CNReporter (void)
 {
 		entries_per_page = 5;
 		n_ent = 0;
 }
 
 
/*
 *		clear - flush yourself
 */
 void CNReporter::clear(void)
 {
 	n_ent = 0;
 }
 
/*
 *		Init_time - initialize.  You give it the number of time steps in your
 *			simulation.  This should get called after all of your metered
 *			CNModelElems have been registered with the CNReporter.
 */
 
 void CNReporter::Init_time (int time)
 {
 		int i;
 		
 		for (i=0; i< n_ent; i++)
 			ent[i]->clear (time);
 		
 		t=0;
 		time_steps = time;
 }
 
 
/*
 *		click - Indicates to the reporter that a time step has passed. Should
 *			be called AFTER a simulation step has finished.
 */
void CNReporter::click (void)
{
		if (t < time_steps) t++;
}


/*
 *		record_signal - Record a signal for the Entity with given id number,
 *			This signal is assumed to be for the current time step.
 */
 
 void CNReporter::record_signal (float in, float out, int id)
 {
 		if (id < 0) return;

 		ent[id]->record (in, out, t);
 }
 
 

/*
 *		Register - register an CNModelElem to have it's simulation data recorded.
 *			Returns an id (index) to be associated with that element.
 */
 
 int CNReporter::Register (CNModelElem *E)
 {
 		if (n_ent == MAX_ENTITIES) return -1;
 		
 		ent[n_ent] = new CNReportedEntity;
 		ent[n_ent]->Init (E);
 		
 		n_ent++;
 		
 		return n_ent -1;
 }
 
 
 
/*
 *		print - Print the simulation results to a file.  Returns flag indicating
 *			if write was okay.
 */
 
 int CNReporter::print (char *filename)
 
{
		FILE *f;
		int i;
		int j;
		int cur_t;
		
/*
 *		open file 
 */
 		if (! (f = fopen (filename,"w"))) return 0;
 		
 /*
  *		write element headers.
  */
  		for (i=0; i< n_ent; i++)
  			ent[i]->write_self (f);
  			
 /*
  *		write your data, a page at a time 
  */
  		for (j = 0; j < n_ent; j+= entries_per_page) {
 /*
  *			write data headers
  */
  			fprintf (f,"time  ");
  			for (i=0; (i < entries_per_page) && (j + i  < n_ent) ;i++)
  				fprintf (f, "%15s ", ent[i+j]->get_name());
  				
  			fprintf (f,"\n         ");
  				
  			for (i=0; (i < entries_per_page) && (j + i  < n_ent) ;i++)
  				fprintf (f, "   IN     OUT     ");
  				
  			fprintf (f, "\n");
  	
 /* 
  *			write your data
  */
  			for (cur_t =0; cur_t < t; cur_t++) {
  				fprintf (f,"%4.1f  ", cur_t * CNSimulator::ms_per_tick()); 
  				for (i=0; (i < entries_per_page) && (j + i  < n_ent) ;i++)
  					fprintf (f, "%7.2f %7.2f ", ent[i+j]->get_in(cur_t), 
  						ent[i+j]->get_out(cur_t));
  				fprintf (f,"\n");
  			}
  		}
  		
  		fclose (f);
  	
  		return 1;
  }
  			
