/*
 *
 *	Biological Neural Net Simulator based on the Bon thesis
 *
 *	Object: CNReporter
 *
 *	This class represnts a package that is used for reporting the
 *	output of a given simulation.  In it's current form, it is pretty
 *	crude, only producing ASCII output, but in theory, it could be
 *	enhanced to produce outputs in other formats.
 *
 */
 
 #pragma once
 
 #include <CObject.h>
 
 #define MAX_ENTITIES 10
 #define MAX_TIME_STEPS 2010
  
 #include <stdio.h>
 
 class CNModelElem;
 
 typedef struct blip {
 	float in;
 	float out;
 } Blip;
 
 class CNReportedEntity  : public CObject {
 	CNModelElem		*elem;
 	Blip			signals[MAX_TIME_STEPS];
 public:
 	void Init (CNModelElem *E);
 	
 	void clear (int time);
 	void record (float in, float out, int t);
 	float get_in (int t);
 	float get_out (int t);
 	const char *get_name(void);
 	void write_self(FILE *f);
 };
 
 class CNReporter : public CObject {
 	CNReportedEntity	*ent[MAX_ENTITIES];   /* array of Entities being reported on */
 	int				t;                    /* current time step */
 	int				n_ent;				  /* number of entities being reported on */
 	int				entries_per_page;	  /* number of entries that will fit on page */
 	int				time_steps;			  /* number of time steps in your simulation */
 public:
 	CNReporter (void);
 	void Init_time (int time);
 	
 	void clear (void);
 	void click (void);
 	void record_signal (float in, float out, int id);
 	int Register (CNModelElem *E);
 	int print (char *filename);
 };
 