/*
 *
 *	Biological Neural Net Simulator based on the Bon thesis
 *
 *	methods for Object: Simulator
 *
 */
 
 
#include "CNSimulator.h"
#include "CNReporter.h"
#include "CNNodeModel.h"
#include "CNModelElem.h"
#include "CNInputModel.h"

/*
 *		class member definitions.
 */
 
float CNSimulator::milsecs = 1.0;
CNReporter *CNSimulator::reporter = 0;



/*
 *	simulation_step - Perform a single time step of the simulation on 
 *	               a network.
 */
void CNSimulator::simulation_step(void)
{
	CNNodeModel *N;
	CNModelElem *E;
	CNInputModel *I;
	

/* 
 *	Report that a time step went by
 */
 	reporter->click();


/*
 *	fire all your input cells and trnsfer their output to the
 *	connected element's input.
 */
	
	CNInputModel::fire_inputs();
	

/*
 *
 *	process each of your elements (determine output potentials
 *	for this time step).
 *
 */
	CNModelElem::process_elements();
	
	
/*
 *	lastly, transfer the output on elements to inputs of other
 *	elements via nodes to prepare for next time step.
 */
	CNNodeModel::do_node_transfers();
		  

}




/*
 *		ms_per_tick - Returns the number of milliseconds per tick
 */
 
/*static*/float CNSimulator::ms_per_tick (void)  { return milsecs; }




/*
 *		record signal - Record a signal from one of the elements being simulated.
 *			Basically passes the request to your reporter.
 */
 
static void CNSimulator::record_signal (float in, float out, int id)
{
	reporter->record_signal (in, out, id);
}



/*
 *		simulate - Start up a simulation.  You give it the number of time steps, 
 *			the simulation interval, and the filename to place the results.
 */
 
 
void CNSimulator::simulate (int steps, float time_step, char *filename)

{

		CNModelElem *E;
		int i, id;
	
/*
 *		create and set up your reporter if not already done.
 */
 		if (!reporter) reporter = new CNReporter;
 		reporter->clear();
 		
 		
/*
 *		prep all of your Elements and your reporter for simulation
 */
		milsecs = time_step;
		CNModelElem::simulation_prep (reporter);
		CNInputModel::simulation_prep ();	
		reporter->Init_time(steps);
	
/*
 *	run a simulation. and print your results
 */
		for (i=0; i< steps; i++) 
			simulation_step ();
			
		reporter->print (filename);
}   

