#pragma once

/******************************************************************************
 CNSomaModel.h

		Interface for the NSomaModel Class
		
		Source code for "NervousSystem"
		Copyright  1994 Joe Geigel. All rights reserved.
		All use is dictated by the terms of the "GNU General Public 
		License" (see the file "CNObjectView.c" for details).
		
	 	Biological Neural Net Simulator based on the Bon thesis
	 
		Object: CNSomaModel
	 
	 	A Soma is the body of the neuron and is the part responsible
	 	for a neuron's firing.
	 
	 	Members are arranged in groups involving I/O potentials, threshold
	 	available firing potential, learning and firing. 
	 
	 	Note that CNSomaModel is a subclass of CNModelElem.

 ******************************************************************************/
 
#define _H_CNSomaModel

#include "CNModelElem.h"	

class CSomaSetup;

class CNSomaModel : public CNModelElem {	
private:
 	/* Members having to do with I/O potentials */
 	float		acc_in_pot;		/* accumulated input potential */
 	float		acc_in_decay;	/* accumulated input decay constant */
	float		aid_per_tick;	/* accumulated input decay const PER TICK */
	 	
 	/* Members having to do with soma threshold */
 	float		cur_thresh;		/* current threshold */
 	float		orig_rest_thresh; /* original resting threshold */
 	float		rest_thresh;	/* current resting threshold */
 	float		t_thresh;		/* refractory time between fire and return to resting
 								   threshold */
 	float		max_rest_thresh;/* maximum resting threshold */
 	float		min_rest_thresh;/* minimum resting threshold */
 	float		damping_const;	/* damping constant used to determine exp effect 
 								   on threshold buildup */
 								   
 	/* Learning parameters */
 	float		learn_fire;		/* resting threshold learning coefficient when firing */
 	float		learn_nofire;	/* resting threshold learning coefficient when not firing */
 								  
  	/* Members having to do with available firing potential */
  	float		cur_fire_pot;	/* current firing potential */
  	float		rest_fire_pot;	/* firing potential at rest */
  	float		fire_recvr_const;/* constant that determines how quickly firing 
  								    potential recovers */
  	float		frc_per_tick;	/* fire recovery constant per tick */
 	
 	/* Members having to do with firing */
 	float		t_fire;			/* time since last firing */
 	float		thresh_rise;	/* amplitude of threshold rise after firing */
 	
 	static int	count;			/* used to automatically name new Somas */
 	
 public:
	// Initialize and copy functions
	CNModel *CopyModel (void);
	int INSomaModel ( void );
	
	void GetSomaData (nSomaData *data);
	void SetSomaData (nSomaData *data);
 	
 	// simulation functions
 	void describe_self (FILE *f);
  	void process (void);
 	int prep_for_simulation (void);
};
 
 
