#pragma once

/******************************************************************************
 CNSynapseModel.h

		Interface for the NSynapseModel Class
		
		Source code for "NervousSystem"
		Copyright  1994 Joe Geigel. All rights reserved.
		All use is dictated by the terms of the "GNU General Public 
		License" (see the file "CNObjectView.c" for details).

	 	Biological Neural Net Simulator based on the Bon thesis
	 
	 	Object: Synapse
	 
	 	A synapse as defined in the Bon model.
	 
	 	Note that Synapse is a subclass of CNModelElem.
		
 ******************************************************************************/
 
#define _H_CNSynapse

#include "CNModelElem.h"

#define INHIBITORY 0
#define EXCITORY 1

class CNSynapseModel : public CNModelElem {
private:
 	/* Kind of Synapse */
 	int			type;			/* INHIBITORY or EXCITORY */
 	
	/* Members having to do with I/O potentials */
 	float		acc_in_pot;		/* accumulated input potential */
 	float		acc_in_decay;	/* accumulated input delay constant */
 	float		aid_per_tick;	/* accumulated input decay per tick */
 	
  	/* Members having to do with firing threshold */
 	float		cur_thresh;		/* current threshold */
 	float		orig_rest_thresh; /* original resting threshold */
 	float		rest_thresh;	/* current resting threshold */
 	float		t_thresh;		/* refractory time between fire and return to resting
 								   threshold */
 	float		damping_const;	/* damping constant used to determine exp effect 
 								   on threshold buildup */
 								   
 	/* Members having to do with vesicle count */
 	float		orig_vesc_cnt;	/* original vesicle count */
 	float		cur_vesc_cnt;	/* current vesicle count */
 	float		max_vesc_cnt;	/* maximum vesicle count */
 	float 		min_vesc_cnt;	/* minimum vesicle count */
 	
 	  
  	/* Members having to do with available firing potential */
  	float		cur_fire_pot;	/* current firing potential */
  	float		fire_recvr_const;/* constant that determines how quickly firing 
  								    potential recovers */
  	float		frc_per_tick;	/* fire recovery constant PER TICK */
  	float		fire_fract;		/* fraction of firing potential that gets transferred
  								   during a fire.  Made up of following two params */
  	float		fire_fract_quant;/* quantum term of fire_fract */
  	float		fire_fract_var;  /* fraction of excess input potential that contributes
  								    to fire_fract */
  	float		orig_ffq;		/* original value of fire_fract_quant */
  	float		orig_ffv;		/* original value of fire_fract_var */
  	
 	
 							   
 	/* Learning parameters */
 	float		ves_learn_fire;		/* vesicle count learning coeff when firing */
 	float		ves_learn_nofire;	/* vesicle count learning coeff when not firing */
 	float		ffq_learn_fire;		/* fire_fract_quant learning coeff when firing */
 	float		ffq_learn_nofire;	/* fire_fract_quant learning coeff when not firing */
 	float		ffv_learn_fire;		/* fire_fract_var learning coeff when firing */
 	float		ffv_learn_nofire;	/* fire_fract_var learning coeff when not firing */
 		
 								
 
 	/* Members having to do with firing */
 	float		t_fire;			/* time since last firing */
 	float		thresh_rise;	/* amplitude of threshold rise after firing */
 	
 	static int	count;			/* used to automatically name new Synapses */
 	
 public:
	// Initialize and copy functions
	CNModel *CopyModel (void);
	int INSynapseModel ( void );
	
	void GetSynapseData (nSynapseData *data);
	void SetSynapseData (nSynapseData *data);
 	
 	// simulation functions
 	void describe_self (FILE *f);
  	void process (void);
 	int prep_for_simulation (void);
};

 
