/* zNervousSystemDoc.c -- document methods */
/* Created 9/23/93 12:10 PM by AppMaker */

/*	We recommend that you not modify this module and instead modify		*/
/*	its subclass, NervousSystemDoc.  The 'z' prefix on this module marks	*/
/*	a module which is likely to be regenerated by AppMaker after you	*/
/*	make changes to the user interface.  The modules without the 'z'	*/
/*	prefix will not be regenerated by AppMaker unless you delete them.	*/
/*	Using a separate subclass to override the AppMaker-generated code	*/
/*	lets you regenerate code without losing your hand-coded changes.	*/ 

#include <Commands.h>
#include <Global.h>
#include <CApplication.h>
#include <CBartender.h>
#include <CDataFile.h>
#include <CDesktop.h>
#include <CError.h>
#include <TBUtilities.h>
#include "ResourceDefs.h"
#include "NervousSystemData.h"
#include "SimulationController.h"
#include "MeterSetup.h"
#include "TrainCellController.h"
#include "SingleEventSetup.h"
#include "SingleCycleSetup.h"
#include "CustomEventSetup.h"
#include "CustomCycleSetup.h"
#include "MainWindow.h"
#include "zNervousSystemDoc.h"

extern	CApplication	*gApplication;	/* The application */
extern	CBartender		*gBartender;	/* The menu handling object */
extern	CError			*gError;		/* The error handling object */

/*----------*/
void	ZNervousSystemDoc::INervousSystemDoc	(CApplication	*aSupervisor,
										 Boolean		printable)
{
	inherited::IDocument (aSupervisor, printable);
	itsData = NULL;

} /* INervousSystemDoc */

/*----------*/
void	ZNervousSystemDoc::Dispose		(void)
{
	ForgetObject (itsData);
	itsFile = NULL;		// was disposed by ForgetObject (itsData)
	inherited::Dispose ();

} /* Dispose */

/*----------*/
void	ZNervousSystemDoc::NewFile		(void)
{
	itsData = new CNervousSystemData;
	itsData->INervousSystemData (this);

	BuildWindows ();

	if (itsWindow != NULL) {
		itsWindow->Select ();
	}

} /* NewFile */

/*----------*/
void 	ZNervousSystemDoc::OpenFile		(SFReply	*macSFReply)
{
	Str63			theName;

	itsData = new CNervousSystemData;
	itsData->INervousSystemData (this);
	itsData->SFSpecify (macSFReply);
	itsData->OpenData (fsRdWrPerm);
	itsFile = itsData;

	BuildWindows ();

	itsFile->GetName (theName);
	if (itsWindow != NULL) {
		itsWindow->SetTitle (theName);
		itsWindow->Select ();
	}

} /* OpenFile */

/*----------*/
void	ZNervousSystemDoc::BuildWindows		(void)
{
	CWindow			*mainWindow;
	CDirector		*subWindow;

	mainWindow = new CMainWindow;
	itsWindow = mainWindow;
	((CMainWindow *)mainWindow)->IMainWindow (this, itsData);
	itsMainPane = ((CMainWindow *)mainWindow)->itsMainPane;
	
	
} /* BuildWindows */

/*----------*/
Boolean	ZNervousSystemDoc::DoSave		(void)
{
	if (itsFile == NULL) {
		return (DoSaveFileAs ());
	} else {
		if (itsData->Save ()) {
			dirty = FALSE;
			return (TRUE);
		} else {
			return (FALSE);
		}
	}
} /* DoSave */

/*----------*/
Boolean	ZNervousSystemDoc::DoSaveAs		(SFReply	*macSFReply)
{
	if (itsData->SaveAs (macSFReply)) {
		itsFile = itsData;
		if (itsWindow != NULL) {
			itsWindow->SetTitle (macSFReply->fName);
		}
		dirty = FALSE;
		return (TRUE);
	} else {
		return (FALSE);
	}

} /* DoSaveAs */

/*----------*/
void	ZNervousSystemDoc::DoRevert		(void)
{
	itsData->Revert ();
	dirty = FALSE;

} /* DoRevert */

/*----------*/
/* This code should be in NervousSystemDoc.c rather than in this file. */
/* AppMaker generated it here because NervousSystemDoc.c already existed. */
/* AppMaker doesn't overwrite an existing non-Z file because it */
/* might have lots of user hand-written code. */
/*----------*/
void	ZNervousSystemDoc::UpdateMenus		(void)
{
	inherited::UpdateMenus ();
	gBartender->EnableCmd (cmdStimulationCell);
	gBartender->EnableCmd (cmdDendrite);
	gBartender->EnableCmd (cmdSoma);
	gBartender->EnableCmd (cmdAxon);
	gBartender->EnableCmd (cmdSynapse);
	gBartender->EnableCmd (cmdNewMeter);
	gBartender->EnableCmd (cmdHideMeters);
	gBartender->EnableCmd (cmdSimulationSetup);
	gBartender->EnableCmd (cmdRunSimulation);
	gBartender->EnableCmd (cmdKillSimulation);
	gBartender->EnableCmd (cmdMeterSetup);
	gBartender->EnableCmd (cmdStimCellSetup);
	gBartender->EnableCmd (cmdSingleEventSetup);
	gBartender->EnableCmd (cmdSingleCycleSetup);
	gBartender->EnableCmd (cmdCustomEventSetup);
	gBartender->EnableCmd (cmdCustomCycleSetup);
	gBartender->EnableCmd (cmdx005);
	gBartender->EnableCmd (cmdx0075);
	gBartender->EnableCmd (cmdx01);
	gBartender->EnableCmd (cmdx025);
	gBartender->EnableCmd (cmdx05);
	gBartender->EnableCmd (cmdx075);
	gBartender->EnableCmd (cmdx1);
	gBartender->EnableCmd (cmdx25);
	gBartender->EnableCmd (cmdx5);
	gBartender->EnableCmd (cmdx75);
	gBartender->EnableCmd (cmdx10);
	
} /* UpdateMenus */

/*----------*/
void	ZNervousSystemDoc::DoCommand		(long		theCommand)
{
	switch (theCommand) {
		case cmdStimulationCell:
				/* handle StimulationCell choice; */
			break;
		case cmdDendrite:
				/* handle Dendrite choice; */
			break;
		case cmdSoma:
				/* handle Soma choice; */
			break;
		case cmdAxon:
				/* handle Axon choice; */
			break;
		case cmdSynapse:
				/* handle Synapse choice; */
			break;
		case cmdNewMeter:
				/* handle NewMeter choice; */
			break;
		case cmdHideMeters:
				/* handle HideMeters choice; */
			break;
		case cmdSimulationSetup:
				//DoSimulationSetup (this);
			break;
		case cmdRunSimulation:
				/* handle RunSimulation choice; */
			break;
		case cmdKillSimulation:
				/* handle KillSimulation choice; */
			break;
		case cmdMeterSetup:
				DoMeterSetup (this);
			break;
		case cmdStimCellSetup:
				//DoStimCellSetup (this);
			break;
		case cmdSingleEventSetup:
				DoSingleEventSetup (this);
			break;
		case cmdSingleCycleSetup:
				DoSingleCycleSetup (this);
			break;
		case cmdCustomEventSetup:
				DoCustomEventSetup (this);
			break;
		case cmdCustomCycleSetup:
				DoCustomCycleSetup (this);
			break;
		case cmdx005:
				/* handle x005 choice; */
			break;
		case cmdx0075:
				/* handle x0075 choice; */
			break;
		case cmdx01:
				/* handle x01 choice; */
			break;
		case cmdx025:
				/* handle x025 choice; */
			break;
		case cmdx05:
				/* handle x05 choice; */
			break;
		case cmdx075:
				/* handle x075 choice; */
			break;
		case cmdx1:
				/* handle x1 choice; */
			break;
		case cmdx25:
				/* handle x25 choice; */
			break;
		case cmdx5:
				/* handle x5 choice; */
			break;
		case cmdx75:
				/* handle x75 choice; */
			break;
		case cmdx10:
				/* handle x10 choice; */
			break;
		
		default:
				inherited::DoCommand (theCommand);
			break;
	} /* switch */

} /* DoCommand */

/* zNervousSystemDoc.c */
